#ifndef __chips_SparseMem_H
#define __chips_SparseMem_H
/*
 * Copyright (c) 1998 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: SparseMem.h,v 1.1 1998/01/19 02:34:16 steve Exp $"

/*
 * This template provides a means of arranging a memory array sparsely
 * in memory. The STRIDE is the number of bytes needed to go from one
 * item to the next. The idx used by the various methods is the
 * element number (not the offset) of the desired item.
 *
 * The implementation provided here presumes that the type T fits in
 * the STRIDE an even number of times.
 */

template <class T, unsigned STRIDE> class SparseMem {

    public:
      SparseMem(T*base)  : base_(base) { }

      T get_value(unsigned idx) const
	    { return base_[idx * STRIDE / sizeof(T)]; }

      void set_value(unsigned idx, T value)
	    { base_[idx * STRIDE / sizeof(T)] = value; }

    private:
      T*const base_;
};

/*
 * $Log: SparseMem.h,v $
 * Revision 1.1  1998/01/19 02:34:16  steve
 *  Add new chip drivers
 *
 */
#endif
