#ifndef __chips_XC4000_H
#define __chips_XC4000_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: XC4000.h,v 1.2 1997/04/14 05:43:10 demo Exp $"
/*
 * $Log: XC4000.h,v $
 * Revision 1.2  1997/04/14 05:43:10  demo
 *  Update init to support rev 3 ISE boards.
 *
 * Revision 1.1  1996/11/05 00:53:27  steve
 *  Add an XC40xx programmer driver.
 *
 */

/*
 * This class supports the configuration interface of Xilinx XC4000
 * FPGA chips. This driver simulates the master of a programming chain
 * and drives the CCLK and DIN pins according to an image passed by
 * the application. The connected device may be a single part or a
 * series of devices, all chained together.
 *
 * The configuration image must be byte aligned, and the driver will
 * write a byte multiple of bits into the configuration stream.
 *
 * For details of parts and the configuration protocol see
 *	XC4000 Series
 *	Field Programmable Gate Arrays
 * available from Xilinx, at <http://www.xilinx.com>.
 *
 * Normally, this driver is not needed once the device(s) is
 * programmed. There is currently no provision for reconfiguring a
 * device once it has been loaded.
 */

# include  <stddef.h>

class XC4000 {

    public:
	// This is the address of the single register that contains
	// the DIN and CCLK bits for the chip. (bits are given as bit
	// numbers, 0 for the lowest, 7 for the highest.)
      explicit XC4000(volatile unsigned long*reg,
		      unsigned din, unsigned cclk, unsigned long m = 0);
      ~XC4000();

	// With this method, the entire configuration image for the
	// daisy-chain of devices is loaded all at once.
      void load_config(const void*bytes, size_t size);

    private:
      volatile unsigned long*const reg_;
      const unsigned long mask_;
      const unsigned din_, cclk_;

    private: // Not implemented
      XC4000(const XC4000&);
      XC4000& operator= (const XC4000&);
};

#endif
