#ifndef __chips_ZR36050_H
#define __chips_ZR36050_H
/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: ZR36050.h,v 1.3 1998/01/21 00:36:14 steve Exp $"

# include  <stddef.h>

/*
 * This class is a driver for the ZR36050 JPEG Image Compression Processor.
 * For complete documentation of the chip, see the ZR36050 data sheet
 * available from ZORAN Compression Solutions <http://www.zoran.com>.
 *
 * All the registers are presumed to be mapped into memory, with a
 * stride that reflects the gap between the registers.
 */

class ZR36050 {

    public:
	// The base is the address of the first register. The stride
	// is the amount added to a register address to get to the
	// next register. With this information, the class can find
	// all the registers.
      explicit ZR36050(volatile unsigned char*base, unsigned stride =1)
      : base_(base), stride_(stride)
       { }

    public:
	// STart a compression cycle.
      void go() { set_register(0, 0); }

	// Calculate the size of the generated compressed image, using
	// the marker information in my memory.
      size_t compressed_size() const;

      enum STATUS0 { SOS=0x01 };
      enum STATUS1 { RFD=0x08, END=0x04 };

      void set_int_bit(STATUS0 bit) { base_[7*stride_] |= bit; }
      void set_int_bit(STATUS1 bit) { base_[8*stride_] |= bit; }

      void clr_int() { base_[7*stride_] = 0; base_[8*stride_] = 0; }
      void clr_int_bit(STATUS0 bit) { base_[7*stride_] &= ~bit; }
      void clr_int_bit(STATUS1 bit) { base_[8*stride_] &= ~bit; }

    public:
	// These are generic chip register access methods. These treat
	// all the registers as an array of 1024 bytes.

      unsigned char get_register(unsigned addr) const
	    { return base_[addr * stride_]; }
      void set_register(unsigned addr, unsigned char val)
	    { base_[addr * stride_] = val; }

    private:
      volatile unsigned char*const base_;
      const unsigned stride_;

    private: // not implemented
      ZR36050(const ZR36050&);
      ZR36050& operator= (const ZR36050&);
};

/*
 * $Log: ZR36050.h,v $
 * Revision 1.3  1998/01/21 00:36:14  steve
 *  tst_status wont work.
 *
 * Revision 1.2  1998/01/20 05:08:08  steve
 *  Add atatus manipulation
 *
 * Revision 1.1  1998/01/19 02:34:17  steve
 *  Add new chip drivers
 *
 */
#endif
