#ifndef __i960_I960Timer_H
#define __i960_I960Timer_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: I960Timer.h,v 1.3 1996/10/28 03:45:15 steve Exp $"
/*
 * $Log: I960Timer.h,v $
 * Revision 1.3  1996/10/28 03:45:15  steve
 *  License notice.
 *
 * Revision 1.2  1996/09/22 16:27:11  steve
 *  I960Timer takes interrupt vector as parameter.
 *
 * Revision 1.1  1996/05/29 21:41:53  steve
 *  Add the I960Timer class.
 *
 */

/*
 * Some i960 architectures support integrated timers. This class
 * provides an interface to them. There may be 0 or more, depending on
 * the type of chip. The location is calculated from the timer
 * instance number (which) so no pointer is required.
 *
 * Also, this timer is a kind of Ticker (a uCR concept) so can be
 * passed to things that expect tickers.
 */

# include  <ucr/Ticker.h>


class I960Timer  : public Ticker {

	// The timer is initialized to use the 1X clock as a tick, so
	// the ns_per_tick value is exactly 1/bus clock.
    public:
      I960Timer(unsigned which, unsigned vector, unsigned long interval);
      ~I960Timer();
      void start();

    private:
      struct REGS_ {
	    volatile unsigned long trr;
	    volatile unsigned long tcr;
	    volatile unsigned long tmr;
      };

      REGS_*const regs_;

      static void isr_(I960Timer*, unsigned);

    private: // not implemented
      I960Timer(const I960Timer&);
      I960Timer& operator= (const I960Timer&);
};


#endif
