#ifndef __mc68322_Band_H
#define __mc68322_Band_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 */
#ident "$Id: Band.h,v 1.5 1995/11/07 07:54:47 steve Exp $"
/*
 * $Log: Band.h,v $
 * Revision 1.5  1995/11/07 07:54:47  steve
 *  Support optional smart-toner mode.
 *
 * Revision 1.4  1995/10/25 04:04:48  steve
 *  Support Automatic printer resolution switching.
 *
 * Revision 1.3  1995/10/17 18:28:08  steve
 *  Put in the band queue class help for
 *  doing queue synchronization.
 *
 * Revision 1.2  1995/10/13 18:43:39  steve
 *  Document bit vs. byte addresses.
 *
 * Revision 1.1  1995/09/02 18:27:07  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */


/*
 * The Band represents a horizontal strip of image. It is really
 * nothing more then a bit block with hooks for enqueuing. The bits
 * pointer is always a *byte* pointer to the first bit, which always
 * starts on a byte boundry. The width and height are in pixels. The
 * depth is the number of bits per pixel.
 *
 * The geometry is a means of specifying the reference plane. It is
 * used by the PrintEngine to select page dimensions and horizontal
 * resolution. The default is zero.
 *
 * NOTE: There is no warp. All the devices that use bands assume that
 * the pixels are PACKED, and there is no padding on the ends of lines.
 */
class BandQueue;

class Band {

    public:
      Band() : geometry_(0), end_of_page_(1), smart_toner_(0), next_(0) { }

      void set(void*b,
	       unsigned short wid, unsigned short hei,
	       unsigned short dep)
	{ bits_ = b; width_ = wid; height_ = hei; depth_ = dep; }

      unsigned short width() const { return width_; }
      unsigned short height()const { return height_; }
      unsigned short depth() const { return depth_; }
      void* bits() const { return bits_; }

      bool end_of_page() const { return end_of_page_; }
      void end_of_page(bool flag) { end_of_page_ = flag; }

      unsigned char geometry()const { return geometry_; }
      void geometry(unsigned char geo) { geometry_ = geo; }

      bool smart_toner() const    { return smart_toner_; }
      void smart_toner(bool flag) { smart_toner_ = flag; }

    private:
      void* bits_;
      unsigned short width_;
      unsigned short height_;
      unsigned short depth_       : 8;
      unsigned short geometry_    : 4;
      unsigned short end_of_page_ : 1;
      unsigned short smart_toner_ : 1;

      friend class BandQueue;
      Band*next_;

    private: // Not implemented
      Band(const Band&);
      Band& operator= (const Band&);
};


class BandQueue {

    public:
      BandQueue();

      void enqueue(Band*);
      Band*pull();
      Band*peek() const;

      unsigned count() const { return count_; }

        // This may prove handy with the ISync class.
      static bool not_empty(const BandQueue*);

    private:
      Band* last_;
      unsigned count_;

    private:
      BandQueue(const BandQueue&);
      BandQueue& operator= (const BandQueue&);
};


#endif
