#ifndef __mc68322_External_H
#define __mc68322_External_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: External.h,v 1.2 1996/10/28 03:45:17 steve Exp $"
/*
 * $Log: External.h,v $
 * Revision 1.2  1996/10/28 03:45:17  steve
 *  License notice.
 *
 * Revision 1.1  1995/10/04 23:17:53  steve
 *  Add the External class.
 *
 */

/*
 * SEE THE MC68322 Integrated Processor User's Manual, section 5.2
 * (pp 5-4 through 5-6) for details on EXTERNAL INTERRUPTS, the
 * subject of this class.
 *
 * This class handles the interrupt register for generic external
 * interruption sources. The interrupts are managed here because many
 * drivers may share each interrupt handler and it is best to factor
 * out this, so that the devices do not step on each other's toes.
 *
 * The programmer can create only one instance of this class per
 * interrupt input (0 or 1). The programmer must tell what the level
 * of the line means, whether active high or active low, edge or
 * level, so that I can program the 322 to do the required magic.
 *
 * It is perfectly reasonable to have many devices that use an
 * external interrupt. That is why this class exists: so several
 * devices can share an External instance. It is also reasonable to
 * bury the instance within the device provate data and not share it,
 * if you have the lines to spare.
 *
 * NOTE: This class does not create an interrupt handler to dispatch
 * to the devices. It is still the job of the device to do
 * that. However, the device class should get the level to use from
 * the level() method.
 *
 * NOTE: The clear_interrupt() method might be called by any isr in
 * the chain for the interrupt level. Therefore, the the interrupt
 * pending bit is not valueable and the external hardware should
 * provide its own means of deciding that it interrupted.
 */

class External {

        // Create an External object. Set the interrupt level, and the
	// style of interrupt. Also select which interrupt line it is
	// by passing 0 or 1.
    public:

      enum Polarity { LOW = 0, HIGH = 1, FALL = 2, RAISE = 3 };
      External(unsigned which, unsigned short level, Polarity mode,
	       unsigned long base =0xfff000);


      unsigned short level() const;


        // Enable or disable the interrupt. The enable and disable
	// nest, so that the interrupt will be enabled as long as at
	// least one client needs it. All the other clients must deal
	// with it.
      void enable();
      void disable();

      void clear_interrupt();

    private:

      unsigned enable_nest_;

      struct REGS_ {
	    unsigned short enable;
	    unsigned short status;
	    unsigned short level;
	    unsigned short mode;
	    unsigned short sen;
      };

      REGS_* const regs_;

    private: // Not implemented
      External(const External&);
      External& operator= (const External&);
};

#endif
