#ifndef __stdio_H
#define __stdio_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: stdio.h,v 1.9 1997/08/02 04:28:58 steve Exp $"
/*
 * $Log: stdio.h,v $
 * Revision 1.9  1997/08/02 04:28:58  steve
 *  Some standard constants.
 *
 * Revision 1.8  1997/04/30 16:53:15  steve
 *  Add initial interrupt support to BIOS32,
 *  add some more ISO C library functions to ucr.
 *
 * Revision 1.7  1996/10/29 23:41:40  steve
 *  Add an implementation of calloc. This just calls
 *  malloc and clears the buffer.
 *
 *  Add implementatsion of printf and setbuf. Note
 *  that printf uses standard streams. I make the standard
 *  streams extern so that the compiler finds them. This
 *  allows targets that have the means to support those streams.
 *
 *  Add the exit() function, which is C and C++ standard.
 *  The _exit() function, which is POSIX, is still needed
 *  for the benifit of libgcc.
 *
 * Revision 1.6  1996/10/29 01:58:49  steve
 *  Add fread to libstdio.
 *
 * Revision 1.5  1996/10/28 03:45:09  steve
 *  License notice.
 *
 * Revision 1.4  1996/09/19 01:38:30  steve
 *  Add fgetc and fgets. stdio input works.
 *
 * Revision 1.3  1996/09/17 22:23:00  steve
 *  Add a math library.
 *
 * Revision 1.2  1996/09/16 17:01:41  steve
 *  Add fprintf and sprintf.
 *
 * Revision 1.1  1996/09/14 00:10:14  steve
 *  Add a stdio library.
 *
 */

/* XXXX NOTE: This is a cop-out. stddef.h actually defines too many
 * things so strictly speaking, this is incorrect.
 */
# include  <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Section 4.9.1
 */

typedef struct _FILE FILE;
typedef unsigned long fpos_t;

#ifndef NULL
#define NULL ((void*)0)
#endif

#ifndef EOF
#define EOF (-1)
#endif

/* These are only meaningful as parameters to setvbuf. */
#define _IOFBF 2
#define _IOLBF 1
#define _IONBF 0

/* There is nothing magical about this buffer size, with this
 * implementation of stdio, it is just what the library is compiled
 * with. If you change this number, you must recompile everything that
 * includes this header, INCLUDING THE STDIO LIBRARY.
 */
#ifndef BUFSIZ
#define BUFSIZ 256
#endif

/* Section 7.9.1
 * The standard streams may or may not exist on a particular target
 * system. If they do, it is also possible that not all of them
 * exist. For example, maybe stdout and stderr exist, but not
 * stdin. By declaring them extern like this, their presence becomes a
 * matter for the linker. Also, by defining them this way, then the
 * application can provide its OWN standard streams if desired.
 */
extern FILE*const stdin;
extern FILE*const stdout;
extern FILE*const stderr;

#define SEEK_CUR 0
#define SEEK_END 1
#define SEEK_SET 2

/* Section 4.9.5.1
 * Flush and close the stream, and release resources that have been
 * used to access it.
 */
extern int fclose(FILE*stream);

/* Section 4.9.5.2
 * Flush outstanding bytes to the device of the stream. If the stream
 * pointer is null, flush all opened streams.
 */
extern int fflush(FILE*stream);

/* Section 4.9.5.3
 */
extern FILE *fopen(const char*filename, const char*mode);

/* Section 4.9.5.5 and section 4.9.5.6
 * Set the buffering mode and buffer space for the stream.
 */
extern int setvbuf(FILE*stream, char*buf, int mode, size_t size);

extern void setbuf(FILE*stream, char*buf);
  /* { (void)setvbuf(stream, buf, buf? _IOFBF : _IONBF, BUFSIZ); } */

/* Section 4.9.6.1
 * Formatted output. Use this in preference to the printf function
 * below, because targets may not have the standard streams.
 */
extern int fprintf(FILE*, const char*fmt, ...)
      __attribute__ ((format (printf, 2, 3)));

/* Section 7.9.6.3
 * This function is printf(stdout, ...). Avoid using this because some
 * targets do not have the stdout stream implemented.
 */
extern int printf(const char*fmt, ...)
      __attribute__ ((format (printf, 1, 2)));

/* Section 7.9.6.5
 * Use the format to write into the character array dest.
 */
extern int sprintf(char*dest, const char*fmt, ...)
      __attribute__ ((format (printf, 2, 3)));

extern int sscanf(const char*s, const char*fmt, ...);

/* Section 7.9.7.1
 * Get a character from the stream. Return a single character, or EOF
 * if there is an error or eof.
 */
extern int fgetc(FILE*stream);

/* Section 7.9.7.2
 * Read a nul-terminated string into the buffer. Read up to the end of
 * line (include the \n in the string), EOF or end of the input
 * buffer. In all cases, the nul is included in the input string.
 */
extern char* fgets(char*s, int n, FILE*stream);

/* Section 7.9.7.3
 * Write the character to the stream. Return the character written, or
 * EOF if an error occurs.
 */
extern int fputc(int c, FILE*stream);

/* Section 7.9.7.4
 * Write the string str to the writeable stream, without the
 * terminating null.
 */
extern int fputs(const char*str, FILE*stream);

/* Section 7.9.7.5
 * Macro version of fgetc.
 */
# define getc(stream) fgetc(stream)

/* Section 4.9.7.8
 * This is a version of fputc that the standards allows to be
 * implemented as a macro. Of course, if it is not that is OK too.
 */
static inline int putc(int c, FILE*stream) { return fputc(c, stream); }


/* Section 7.9.8.1
 * Read an array of objects to the stream. Each object has size
 * (size) and there are (nmemb) of them. The result is the number of
 * elements read.
 */
extern size_t fread(void*ptr, size_t size, size_t nmemb, FILE*stream);


/* Section 7.9.8.2
 * Write an array of objects to the stream. Each object has size
 * (size) and there are (nmemb) of them.
 */
extern size_t fwrite(const void*ptr, size_t size, size_t nmemb, FILE*stream);


#ifdef __cplusplus
}
#endif

#endif
