#ifndef __ucr_stdlib_H
#define __ucr_stdlib_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: stdlib.h,v 1.10 1997/07/31 22:48:27 steve Exp $"
/*
 * $Log: stdlib.h,v $
 * Revision 1.10  1997/07/31 22:48:27  steve
 *  Add more ISO-C functions.
 *
 * Revision 1.9  1997/04/30 16:53:15  steve
 *  Add initial interrupt support to BIOS32,
 *  add some more ISO C library functions to ucr.
 *
 * Revision 1.8  1997/04/25 18:45:51  steve
 *  More ISO C support.
 *
 * Revision 1.7  1996/10/29 23:41:40  steve
 *  Add an implementation of calloc. This just calls
 *  malloc and clears the buffer.
 *
 *  Add implementatsion of printf and setbuf. Note
 *  that printf uses standard streams. I make the standard
 *  streams extern so that the compiler finds them. This
 *  allows targets that have the means to support those streams.
 *
 *  Add the exit() function, which is C and C++ standard.
 *  The _exit() function, which is POSIX, is still needed
 *  for the benifit of libgcc.
 *
 * Revision 1.6  1996/10/28 03:45:10  steve
 *  License notice.
 *
 * Revision 1.5  1996/07/30 03:18:06  steve
 *  Add the string.h header, and strlen.
 *
 * Revision 1.4  1996/05/27 22:42:42  steve
 *  Describe the exit() call.
 *
 * Revision 1.3  1995/10/17 19:10:45  steve
 *  Create proper stddef.h header file.
 *
 * Revision 1.2  1995/09/03 05:27:21  steve
 *  Add some defaults for fault handlers.
 *  Add more ANSI standard support into the stdlib.h
 *  header file, and add the strtoul() routine to
 *  the library.
 *
 * Revision 1.1  1995/09/02 18:27:06  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

/*
 * This header file is required by the hosted ANSI C standard. It
 * contains types and prototypes for standard functions. Not
 * everything that the standard requires of this file is supported by
 * the environment.
 *
 * Section numbers in comments below refer to the document:
 *	American National Standard for Information Systems --
 *		Programming Language C
 * ANSI/ISO 9899-1990
 */

#if defined(__cplusplus)
extern "C" {
#endif

# include  <stddef.h>

/* String conversion functions, per 7.10.1
 */
extern double strtod(const char*nptr, char**endptr);
extern long strtol(const char*nptr, char**endptr, int base);
extern unsigned long strtoul(const char*nptr, char**endptr, int base);

extern inline int atoi(const char*str)
      { return (int)strtol(str, 0, 10); }
extern inline int atol(const char*str)
      { return strtol(str, 0, 10); }


/* Memory management, per 7.10.3
 */
extern void* calloc(size_t nmemb, size_t size);
extern void  free(void*);
extern void* malloc(size_t size);
/* extern void* realloc(void*ptr, size_t size); */


/* Communication with the Environemnt, per 7.10.4
 * Not all of these are necessarily supported on every
 * platform. However, abort() is sure to exist everywhere.
 */
extern void abort(void)      __attribute__ ((__noreturn__));
extern void exit(int status) __attribute__ ((__noreturn__));

# define EXIT_SUCCESS 0
# define EXIT_FAILURE -1

/* Searching and sorting, per 7.10.5
 */

extern void qsort(void*base, size_t nmemb, size_t size,
		  int (*compar)(const void*, const void*));

#if defined(__cplusplus)
}
#endif

#endif
