#ifndef __ucr_ISema_H
#define __ucr_ISema_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ISema.h,v 1.3 1998/04/02 02:44:18 steve Exp $"
/*
 * $Log: ISema.h,v $
 * Revision 1.3  1998/04/02 02:44:18  steve
 *  test const, add peek().
 *
 * Revision 1.2  1996/10/28 03:45:21  steve
 *  License notice.
 *
 * Revision 1.1  1996/07/18 23:34:19  steve
 *  Add a semaphore class.
 *
 */

# include  <ucr/ISync.h>

/*
 * This class implements the classic Dijkstra semaphore, a counting
 * semaphore that can be decremented or incremented. It blocks threads
 * that attempt to decrement past zero. There is no maximum value for
 * the semaphore, so incrementing will always succeed.
 */
class ISema  : public ISync {

    public:
      ISema(unsigned initial = 1);
      ~ISema();

    public:
	// Subtract 1 from the current semaphore value. If the value
	// is already zero, block until some other thread (or ISR)
	// pushes the value high enough that the decr can complete.
      void decr();

	// Add one to the semaphore value. If any threads are blocked
	// waiting for a decrement, a blocked thread will be released.
      void incr();

	// This is a convenient way of waiting for the semaphore to
	// become non-zero without decrementing it.
      void test() const;

      unsigned peek() const { return semaphore_; }

    private:
      volatile unsigned semaphore_;

    private: // Not implemented
      ISema(const ISema&);
      ISema& operator= (const ISema&);
};


#endif
