#ifndef __ucr_fault_H
#define __ucr_fault_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: fault.h,v 1.3 1996/10/28 03:45:26 steve Exp $"
/*
 * $Log: fault.h,v $
 * Revision 1.3  1996/10/28 03:45:26  steve
 *  License notice.
 *
 * Revision 1.2  1996/08/14 00:24:36  steve
 *  Add fault information to the THREAD object so that
 *  it can be retrieved at any time.
 *
 * Revision 1.1  1996/08/10 22:06:53  steve
 *  Add generic fault handling code.
 *
 */

/* A lot of this is CPU specific. get it from fault_asm.h. */
# include  <ucr/fault_asm.h>
struct FAULT_RECORD;
struct THREAD;

/*
 * Use this function to set a fault handler for some fault number. Do
 * this and all faults of that number are delivered to the named
 * function, with the fault record passed as the only parameter. The
 * function processes the fault then returns true if the thread may
 * continue, false if the thread must die.
 *
 * The fault numbers depend on the target CPU type, and are described
 * in the fault_asm.h header file for your target. The FAULT_RECORD
 * also depends on the target CPU type and is described there. The
 * FAULT_COUNT defines the range of faults, although there may be
 * holes for reserved faults or other uses.
 *
 * NOTE: The fault number may not necessarily be an index into a fault
 * table. Some CPUs don't have a seperate fault table, so the numbers
 * are defined by the assembly code that receives the faults and
 * assigns numbers. Get the numbers from the fault_asm.h header file.
 */

typedef bool (*ucr_fault_handler)(void*);
extern void set_fault(unsigned num, ucr_fault_handler, void*arg);

/*
 * When in a fault handler, a thread has a fault record associated
 * with it. This function returns a pointer to it. If the thread is
 * not faulted, then this returns 0. At any rate, when a thread
 * returns from the handler for a recoverable fault the fault record
 * goes away.
 */

extern const FAULT_RECORD* get_fault(THREAD*tid);

#endif
