#ifndef __ucr_ucr_H
#define __ucr_ucr_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ucr.h,v 1.12 1996/11/12 00:38:45 steve Exp $"
/*
 * $Log: ucr.h,v $
 * Revision 1.12  1996/11/12 00:38:45  steve
 *  Implement the sparc longjmp, fix the IEEEFP byte
 *  extraction for sparc, add the UCR_STACK_ATTRIBUTE
 *  definition and get the regress test to work for sparc_linux.
 *
 * Revision 1.11  1996/10/28 03:45:27  steve
 *  License notice.
 *
 * Revision 1.10  1996/09/19 04:43:54  steve
 *  Add the i960_ise target, change the target
 *  configuration headers to allow sharing by
 *  targets, fix up the make clean rule.
 *
 * Revision 1.9  1996/08/28 02:09:42  steve
 *  Get rid of some obsolete cruft.
 *
 * Revision 1.8  1996/08/15 02:58:57  steve
 *  Add thread_set_reg.
 *
 * Revision 1.7  1996/08/10 04:05:41  steve
 *  Inline the thread_spl things where possible,
 *  stop basing heap on estack, and fix startup
 *  code to use end_heap.
 *
 * Revision 1.6  1996/08/08 20:57:21  steve
 *  Minor optimizations by inlining a few things
 *  and setting optimization attributes.
 *
 * Revision 1.5  1996/08/08 05:22:39  steve
 *  Get rid of the __Scheduler class, it was not
 *  needed. Add the thread_runnable and _reschedule
 *  functions for thread scheduling.
 *
 * Revision 1.4  1996/08/03 03:30:35  steve
 *  Create the ucr/asm.h header as a target specific
 *  header with line functions, etc; Replace the
 *  __isr_run_flag with a bit of inline assembly code.
 *
 *  Move much thread code out of the __Secheduler class
 *  and into more public thread_ functions, to eliminate
 *  some wasted function call depth. Even the main_thread
 *  is move into its own file.
 *
 *  Keep a list of available threads for debug use, and
 *  add a function to get thread registers.
 *
 * Revision 1.3  1996/05/29 04:28:35  steve
 *  Portable way to ENABLE interrupts.
 *
 * Revision 1.2  1995/10/06 00:07:48  steve
 *  support stack overflow checks.
 */

/*
 * NOTE: The thread manipulation uses the ThreadQueue class to
 * implement a run queue, and synchronization primitives use
 * ThreadQueue objects for holding blocked threads. See the
 * <ucr/ThreadQueue.h> header for important details about thread
 * handling in uCR.
 */


/*
 * These are some inline functions or other target specific
 * definitions that are more then simple parameters. Some assembly
 * code appears here, were it can be inlined.
 */
# include  <ucr/config.h>

/*
 * If the stack attribute is not defined in the config.h, then take
 * the default to align the stack with the size if long integers. The
 * programmer should use __attribute__((UCR_STACK_ATTRIBUTE)) to get
 * the correct alignment for the CPU at hand.
 */
#ifndef UCR_STACK_ATTRIBUTE
#define UCR_STACK_ATTRIBUTE aligned(sizeof(long))
#endif

/*
 * set_isr sets up an interrupt service routine for the interrupt. The
 * vector is the interrupt level to intercept. fun is a pointer to a
 * function taking an application arg, and the vector number (so that
 * an isr can handle many interrupts, if so desired). The arg is
 * passed to the isr every time is is called.
 *
 * Vector meaning depends on the target architecture.
 */
extern void set_isr(unsigned vector, void (*fun)(void*, unsigned),
		    void*arg);


struct THREAD;

  // Create a thread from the stack space passed. Thread status
  // information will be stored onto the top of the stack, and the
  // true stack dimensions will be calculated. Also prepare the stack
  // to be executed. The value returned from this function can be
  // passed to thread_switch. The thread is not in any way started
  // here, but it is available in a global list, so debuggers should
  // notice it.

  // Please align the stack on a full word boundry, as this
  // can improve performance, although 16bit alignment will work
  // fine on some CPUs. Byte alignment is definitely wrong.

  // (Thread safe: it only manipulates its parameters, but for adding
  // the structure to the global thread list, and that is protected.)
extern THREAD* thread_create(void*stack, unsigned size,
			     void (*fun)(void*), void*arg);


/*
 * These two functions implement really the most fundamental thread
 * scheduling functions. A call to thread_runnable() moves a thread to
 * the end of the run queue (or the front) without causing any change
 * of active thread. The thread_reschedule() function looks at the
 * thread in front of the run queue, and if it is different from the
 * current thread, causes a thread switch.
 *
 * In principle, these two, and thread_self() and thread_spl*() can be
 * used to implement all the thread scheduling and synchronization
 * functions of uCR. You can in fact use them to implement your own if
 * you wish. However, I include other things for convenience, and
 * because I can provide slight performance improvements in some cases.
 *
 * Both of these may be called in interrupt handlers, although a
 * thread switch won't actually happen until returning from the ISR.
 */
extern void thread_runnable(THREAD*tid, bool front_flag = false);
extern void thread_reschedule();

  // Activate the specified thread. the thread that calls this
  // function will suspend (but remain runnable) and the specified
  // thread will start executing immediately. In the process of making
  // the thread runnable, it will pull it from any ThreadQueue
  // objects. thread_yield() will switch to the next thread in the run
  // queue.
  //
  // thread_switch is also the way to start a thread created by
  // thread_create.
  //
  // Both of these methods can be called in ISRs. They are ISR safe,
  // although thread_switch() may implicitly manipulate a ThreadQueue
  // if the thread is not in the run queue, which may be unsafe.
extern void thread_switch(THREAD*);
extern void thread_yield();

  // Return a thread id for the currently executing thread. This is
  // ISR safe. In an ISR, thread_self() returns the thread that was
  // interrupted.
inline THREAD* thread_self()
{
      extern struct THREAD* uCR_current_thread;
      return uCR_current_thread;
}


  // Threads may be interrupted by exceptions. In some cases this is
  // not good, so the following routines set the priority level of the
  // CPU so that interrupts are blocked. The thread_spl*() functions
  // return a cookie that reflects the current processor level, which
  // should be restored with thread_spl when you are done.

  // NOTE: These are mentioned in <ucr/asm.h>, even if there we only
  // declare them external. Inlining small functions can be a huge win
  // because they are used a lot.

//extern "C" unsigned short thread_splhi();
//extern "C" unsigned short thread_spl(unsigned short cookie);

  // This method will stop the CPU at a priority that allows
  // interrupts. An interrupt will cause the CPU to resume. This is
  // not the same as thread_yield(). Unfortunately, the pause will
  // stop the whole computer, not just the thread.
extern "C" void thread_pause();


/*
 * Debug tools can use this to get a list of all the existing threads,
 * including thread_self() which always the first in the table. The
 * thread ids are written into the passed array, stopping at the
 * number of threads or the size of the table, whichever is less. The
 * return value is the number of threads returned.
 *
 * The thread_regs function fills the buffer with the contents of the
 * registers for the thread. The registers are arranged in memory to
 * be compatible with the layout in the GDB debuggers.
 */
extern unsigned thread_list(THREAD**table, unsigned ntable);
extern void thread_regs(THREAD*tid, void*buffer);
extern void thread_set_reg(THREAD*tid, unsigned regno, unsigned long val);

/*
 * Get a stack trace, including the location in the current
 * function where stack_trace was called. This returns an array of
 * program counters that you must look up yourself.
 */
extern unsigned stack_trace(unsigned long*buffer, unsigned bcount);

/*
 * Perform some tests on the thread to make sure there isn't a problem
 * somewhere. This is more thorough then the normal sanity checks done
 * during the normal course of events.
 */
extern void thread_self_test(THREAD*);



#endif
