/*
 * Copyright (c) 1996-1998 Picture Elements, Inc.
 *    Stephen Williams <steve@picturel.com>
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: BUSDevice.cc,v 1.11 1998/05/30 18:39:04 steve Exp $"

/*
 * This class implements the device specific parts of the uCR channel
 * protocol. Things that happen at low level, especially interrupt
 * type things, happen here.
 */

# include  <chips/BUS.h>
# include  <assert.h>

# define ROOT_TABLE_BELLMASK 0x01
# define STATUS_BELLMASK     0x02
# define CHANGE_BELLMASK     0x04

BUS::Device::~Device()
{
}

/*
 * This is called by the master to tell me that I am connected. I can
 * notice here that I have received a root table, and I can respond to
 * the host. (The host can ignore the interrupt if it is not pertinent.)
 */
void BUS::Device::device_init(BUS*m)
{
      master_ = m;

      unsigned long addr = get_mailbox(0);
      if (addr) {
	    master_->set_root_table_ptr_(addr);
	    set_mailbox(2, addr);
	    ring_host(0);
      }
}


/*
 * Receiving a root table pointer is synchronous. I get the interrupt
 * saying it is there, I tell the master_ about the new value, and I
 * respond to the host by copying the pointer to the output mailbox
 * and ringing.
 */

void BUS::Device::bus_bell_isr(unsigned long mask)
{
	// Pass the root table change interrupt to the bus.
      if (master_ && mask & ROOT_TABLE_BELLMASK) {
	    unsigned long addr = get_mailbox(0);
	    master_->set_root_table_ptr_(addr);
	    set_mailbox(2, addr);
	    ring_host(0);
      }

      if (master_ == 0)
	    return;

	// Pass the status message word to the bus.
      if (mask & STATUS_BELLMASK) {
	    master_->set_status_message_(get_mailbox(1));
	    ring_host(1);
      }

	// Events about changes in the channel status do not get an
	// acknowledgment, but do get handled by the BUS object.
      if (mask & CHANGE_BELLMASK) {
	    master_->set_channel_event_();
      }

}

/*
 * $Log: BUSDevice.cc,v $
 * Revision 1.11  1998/05/30 18:39:04  steve
 *  Better reset behavior.
 *
 * Revision 1.10  1998/05/26 16:16:32  steve
 *  New channel protocol
 *
 */
