/*
 * Copyright (c) 1998 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: BUScread.cc,v 1.9 1998/05/26 16:16:32 steve Exp $"

# include  <chips/BUS.h>
# include  <assert.h>

/*
 * Read bytes from the host into the passed buffer. This causes a DMA
 * operation to transfer the bytes directly into the caller's
 * buffer. Be careful that file marks are also readable, yet have a
 * size of zero.
 */
bool BUS::Channel::readable() const
{
      if ((roff_ < rsize_) || (rsize_ == 0 && raddr_ > 0))
	    return true;

      master_->check_channel_status_(this);
      return (roff_ < rsize_) || (rsize_ == 0 && raddr_ > 0);
}

bool BUS::Channel::wait_for_raddr_(const volatile void*o)
{
      Channel*obj = (Channel*)o;
      return obj->readable();
}

size_t BUS::Channel::read(void*buffer, size_t count, bool par_flag)
{
      if (read_eof_)
	    return 0;

      assert( master_ );
      unsigned char*buf = (unsigned char*)buffer;
      size_t tcount = 0;

      while (tcount < count) {

	      // Careful not to block if I can return partial results.
	    if (par_flag && tcount && !wait_for_raddr_(this))
		  break;

	    master_->dispatch_sync_.sleep(wait_for_raddr_, this);

	      // Watch out for file marks.
	    if (rsize_ == 0) {
		  read_eof_ = true;
		  master_->consume_out_buffer_(this);
		  return tcount;
	    }

	    size_t trans = count - tcount;
	    if (trans + roff_ > rsize_)
		  trans = rsize_ - roff_;

	    master_->dev_->bus_dma_read(raddr_ + roff_, buf, trans);
	    tcount += trans;
	    buf += trans;
	    roff_ += trans;

	    if (roff_ == rsize_)
		  master_->consume_out_buffer_(this);

      }

      return tcount;
}


/*
 * $Log: BUScread.cc,v $
 * Revision 1.9  1998/05/26 16:16:32  steve
 *  New channel protocol
 *
 */
