/*
 * Copyright (c) 1998 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: BUSframe.cc,v 1.1 1998/06/10 16:53:49 steve Exp $"


# include  <chips/BUS.h>
# include  <assert.h>

/*
 * This is the common frame read/write function. It checks the frame
 * for sanity, and performs the transfer.
 */
void BUS::copy_frame_(unsigned fr, size_t off,
		      void*buf, size_t cnt,
		      bool wr_flag)
{
      char*bp = (char*)buf;
      assert( fr < 16 );

      unsigned long ftable = dev_->readl(root_table_ptr_+8+fr*8 + 0);
      unsigned long fmagic = dev_->readl(root_table_ptr_+8+fr*8 + 4);

      if (fmagic == 0)
	    return;

      if (dev_->readl(ftable+0) != fmagic)
	    return;

      if (dev_->readl(ftable+4) != ftable)
	    return;

      const unsigned long psize = dev_->readl(ftable+8);
      const unsigned long pcnt  = dev_->readl(ftable+12);

      if (off >= (psize * pcnt))
	    return;

      if ((off + cnt) > (psize * pcnt))
	    cnt = (psize * pcnt) - off;

      unsigned poff = off / psize;
      off %= psize;

      while (cnt > 0) {
	    size_t trans = cnt;

	    if (trans + off > psize)
		  trans = psize - off;

	    unsigned long pci = dev_->readl(ftable+16 + poff*4);
	    dev_->bus_transfer(pci + off, bp, trans, wr_flag);

	    poff += (off+trans)/psize;
	    off   = (off+trans)%psize;
	    cnt  -= trans;
	    bp   += trans;
      }

      dev_->bus_transfer_done();
}

/*
 * $Log: BUSframe.cc,v $
 * Revision 1.1  1998/06/10 16:53:49  steve
 *  2way access to frames.
 *
 */

