/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams <steve@picturel.com>
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: I960Timer.cc,v 1.1 1998/07/16 22:51:37 steve Exp $"

# include  <i960/I960Timer.h>
# include  <ucr/ucr.h>
# include  <ucr/param.h>
# include  <assert.h>

# define NS_PER_CLOCK 30

const unsigned long TERMINAL_COUNT = 1;
const unsigned long TIMER_ENABLE   = 2;
const unsigned long TIMER_AUTOLOAD = 4;
const unsigned long TIMER_PROTECT  = 8;

void I960Timer::isr_(I960Timer*obj, unsigned)
{
      obj->handle_tick();
}

I960Timer::I960Timer(unsigned which, unsigned vec, unsigned long interval)
: Ticker(NS_PER_CLOCK*interval), regs_( (REGS_*)(0xff000300 + 0x10*which) )
{
      regs_->tmr = 0;
      regs_->trr = interval;
      regs_->tcr = interval;
      regs_->tmr = TIMER_AUTOLOAD | TIMER_PROTECT;

      switch (which) {
	  case 0:
	  case 1:
	    set_isr(vec, isr_, this);
	    break;
	  default:
	    assert(0);
      }
}

void I960Timer::start()
{
      regs_->tmr = regs_->tmr | TIMER_ENABLE;
}

I960Timer::~I960Timer()
{
      regs_->tmr = 0;
}

/*
 * $Log: I960Timer.cc,v $
 * Revision 1.1  1998/07/16 22:51:37  steve
 *  Finally get rid of libi960.
 *
 */
