/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams <steve@picturel.com>
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: PLX9060p.cc,v 1.9 1998/03/24 03:50:12 steve Exp $"

# include  <chips/BIOS32.h>
# include  <chips/PLX9060p.h>
# include  <assert.h>


PLX9060p::PLX9060p(const BIOS32*p, unsigned char b, unsigned char df)
: pci_(p), bus_(b), dev_fn_(df), dma_base_(0)
{
      unsigned long base;

	// BAR0 for the PLX must be prefetchable 32bit memory.
      pci_->read_config_dword(bus_, dev_fn_, 0x10, base);
      assert( (base & 0xf) == 0 );

      plx_base_ = reinterpret_cast<void*>(base & ~0xf);

	// BAR2 has a region used to access the memory on the other
	// side of the chip. What this memory does is up to the target
	// over there.
      pci_->read_config_dword(bus_, dev_fn_, 0x18, base);
	//assert( (base & 0xf) == 0x8 );

      usr_base_ = reinterpret_cast<void*>(base & ~0xf);


	// Turn it on. (I/O accesses are not used in general.)
      pci_->write_config_dword(bus_, dev_fn_, 0x04, 0xffff0006);
}

PLX9060p::~PLX9060p()
{
      pci_->write_config_word(bus_, dev_fn_, 0x04, 0x0000);
}

/*
 * Arrange for PLX local DMA registers to be available from the PCI
 * side by configuring the expansion rom region to map the registers
 * and locating in in the top half of the local address space 0 BAR.
 */
void PLX9060p::invoke_dma_erom_hack(unsigned long remap)
{
      unsigned long range = dword_(plx_base_, 0);
      unsigned long mask = range & ~0xf, flags = range & 0xf;

	// Divide the existing region in half, and give the low half
	// to the existing stuff, and the high half to the expansion rom.
      mask = (mask >> 1) | 0x80000000;
      dword_(plx_base_, 0)    = mask | flags;
      dword_(plx_base_, 0x10) = mask;

      unsigned long base;
      pci_->read_config_dword(bus_, dev_fn_, 0x18, base);
      base &= ~0xf;

	// Calculate the middle address using the used to cut the
	// range in half. Also, add in the enable bit, while we're at
	// it. This writes into the Expansion ROM Base Address
	// register the calculated address.
      base += 2 + ~mask;
      pci_->write_config_dword(bus_, dev_fn_, 0x30, base);
      dma_base_ = reinterpret_cast<volatile void*>(base & ~0xf);

	// Now configure the bus for DMA operations. Write the local
	// bus address for the DMA registers, and add in the bits
	// needed to configure EROM access to that region.
      dword_(plx_base_, 0x14)  = remap & ~0x7ff;
      dword_(plx_base_, 0x18) |= 0x040b0000;
}

/*
 * $Log: PLX9060p.cc,v $
 * Revision 1.9  1998/03/24 03:50:12  steve
 *  Use default PCI command codes for DMA.
 *
 * Revision 1.8  1998/01/29 22:01:46  steve
 *  Set DMA PCI command.
 *
 * Revision 1.7  1997/11/03 23:23:55  steve
 *  enum for DMA registers.
 *
 * Revision 1.6  1997/08/11 20:09:20  steve
 *  Divide the NVRAM code to different files.
 *
 * Revision 1.5  1997/04/14 06:01:09  steve
 *  Resolve conflict with copied changes.
 *
 * Revision 1.4  1997/04/14 05:40:05  demo
 *  Add support for DMA and words.
 *
 * Revision 1.3  1997/03/28 05:45:45  steve
 *  Add methods to control IO pins.
 *
 * Revision 1.2  1997/03/28 03:23:11  steve
 *  config space IO checks for errors.
 *
 * Revision 1.1  1997/03/27 00:08:56  steve
 *  Add the PLX9060 PCI side driver, and make
 *  the BIOS32 support short word config cycles.
 *
 */
