/*
 * Copyright (c) 1998 Picture Elements, Inc.
 *    Stephen Williams <steve@picturel.com>
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: PLX9080p.cc,v 1.6 1998/06/16 00:32:07 steve Exp $"

# include  <chips/BIOS32.h>
# include  <chips/PLX9080p.h>
# include  <assert.h>


PLX9080p::PLX9080p(const BIOS32*p, unsigned char b, unsigned char df)
: pci_(p), bus_(b), dev_fn_(df)
{
      unsigned long base;

	// BAR0 for the PLX must be prefetchable 32bit memory.
      pci_->read_config_dword(bus_, dev_fn_, 0x10, base);
      assert( (base & 0xf) == 0 );

      plx_base_ = reinterpret_cast<void*>(base & ~0xf);

	// BAR2 has a region used to access the memory on the other
	// side of the chip. What this memory does is up to the target
	// over there.
      pci_->read_config_dword(bus_, dev_fn_, 0x18, base);
      usr0_base_ = reinterpret_cast<void*>(base & ~0xf);

      pci_->read_config_dword(bus_, dev_fn_, 0x1c, base);
      usr1_base_ = reinterpret_cast<void*>(base & ~0xf);

	// Turn it on. (I/O accesses are not used in general.)
      pci_->write_config_dword(bus_, dev_fn_, 0x04, 0xffff0006);

	// Force the endian swap register to known state.
      dword_(plx_base_, 0x0c) = 0;
}

PLX9080p::~PLX9080p()
{
      pci_->write_config_word(bus_, dev_fn_, 0x04, 0x0000);
}

void PLX9080p::dma0_stop()
{
      dma0_cmd_status(0);
      if (! (dma0_cmd_status()&0x10)) {
	    dma0_cmd_status(0x0c);
	    while (! (dma0_cmd_status()&0x10))
		  ; /* busy wait */
      }
}

void PLX9080p::dma1_stop()
{
      dma1_cmd_status(0);
      if (! (dma1_cmd_status()&0x10)) {
	    dma1_cmd_status(0x0c);
	    while (! (dma1_cmd_status()&0x10))
		  ; /* busy wait */
      }
}

/*
 * $Log: PLX9080p.cc,v $
 * Revision 1.6  1998/06/16 00:32:07  steve
 *  Add stop for dma1
 *
 * Revision 1.5  1998/06/11 19:07:12  steve
 *  Add a DMA stop method.
 *
 * Revision 1.4  1998/06/06 01:51:12  steve
 *  Support for endian swapping.
 *
 * Revision 1.3  1998/06/03 17:36:21  steve
 *  Add Access to local bus retion 1.
 *
 * Revision 1.2  1998/05/02 23:36:26  steve
 *  Allow default PCI command codes.
 *
 * Revision 1.1  1998/03/05 03:08:10  steve
 *  Add the PLX9080p class.
 *
 */
