/*
 * Copyright (c) 1998 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: PM2m.cc,v 1.4 1998/06/30 20:28:45 steve Exp $"

# include  <chips/PM2m.h>

PM2m::PM2m(volatile unsigned short&index, volatile unsigned short&data)
: index_(index), data_(data)
{
      write_(CTL_MASTER, 0);
      write_(CTL_ENCODE, 0);
      write_(CTL_JBIG, 0);
      write_(ENC_CODE, 0);
      write_(CTL_ROT_SCALE, 0);
      write_(CTL_KREG_FILL, 0);
      write_(CTL_INT_DMA,   0);
      write_(CTL_SPECIAL, 0);
      write_(X_SCALE, 256);
      write_(Y_SCALE, 256);
      write_(INIT_X_SCALE, 256);
      write_(INIT_Y_SCALE, 256);
}

PM2m::~PM2m()
{
      write_(CTL_MASTER, 0);
}

void PM2m::pass_thru()
{
      set_clr_(CTL_MASTER, COLOR_OUT, 0x0001|HF_ENC|HF_DEC|EN_SCR);
}

void PM2m::huff_encode(HE_MODE mode, unsigned short k)
{
      set_clr_(CTL_MASTER, 0, 0x0001|HF_ENC|HF_DEC|EN_SCR);
      write_(CTL_KREG_FILL, k? ((k-1)<<8) : 0);
      write_(CTL_DECODE, 0);
      write_(CTL_ENCODE, mode);
      set_clr_(CTL_MASTER,
	       HF_ENC|OUTPUT_BYTE_FLIP|COLOR_IN,
	       HF_DEC|EN_SCR);
}

void PM2m::geometry_in(unsigned short width, unsigned short height)
{
      write_(IP_CLIP_OFF, 0);
      write_(IN_PIXELS, width);
      write_(IP_CLIP_PAD, 0);
      write_(IL_CLIP_OFF, 0);
      write_(IN_LINES, height);

      write_(PIXELS, width);
      write_(LINES, height);
}

void PM2m::geometry_out(unsigned short width, unsigned short height)
{
      write_(OP_CLIP_OFF, 0);
      write_(OP_CLIP_EXT, width);
      write_(OL_CLIP_OFF, 0);
      write_(OL_CLIP_EXT, height);

      write_(OL_PAD_TM, 0);
      write_(OL_PAD_BM, 0);
      write_(OP_PAD_LM, 0);
      write_(OP_PAD_RM, 0);
      write_(PAD_CONTROL, 0);
}

void PM2m::configure_sram(SizeCode size, bool fast, bool wide)
{
      unsigned short set = 0, clr = 0;

      if (fast) set |= FAST_RAM;
      else      clr |= FAST_RAM;

      if (wide) set |= WORD_RAM;
      else      clr |= WORD_RAM;

      set_clr_(CTL_ENH, set, clr);

      set = size << 12;
      clr = 0x7000 & ~(size << 12);
      set_clr_(CTL_ROT_SCALE, set, clr);
}

void PM2m::set_burst_dma_mode(BurstCode in, BurstCode out)
{
      set_clr_(CTL_MASTER, 0, BYTE_IN|BYTE_OUT);
      set_clr_(CTL_INT_DMA, I_DMA_EN|O_DMA_EN|I_STROBE_EN|O_STROBE_EN, 0);
      set_clr_(CTL_SPECIAL, 0, I_FIFO_EN|O_FIFO_EN);
      write_(INPUT_CTL,  0);
      write_(OUTPUT_CTL, 0);
      write_(CTL_FIFO, in | (out<<4) | (out==BURST_NONE? 0 : 0x200));
}

void PM2m::configure_int(bool done, bool error, bool raw)
{
      unsigned short set = 0, clr = 0;

      if (done) set |= EN_DONE;
      else      clr |= EN_DONE;

      if (error) set |= EN_ERROR;
      else       clr |= EN_ERROR;

      if (raw) set |= EN_RAW;
      else     clr |= EN_RAW;

      set_clr_(CTL_INT_DMA, set, clr);
}

/*
 * $Log: PM2m.cc,v $
 * Revision 1.4  1998/06/30 20:28:45  steve
 *  Make sure chip is disabled.
 *
 * Revision 1.3  1998/06/17 01:30:41  steve
 *  More compression types.
 *
 * Revision 1.2  1998/06/16 00:32:44  steve
 *  Proper reset in front of ccitt compression.
 *
 * Revision 1.1  1998/03/11 18:22:01  steve
 *  Add PM2m driver class.
 *
 */

