/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: SC2692.cc,v 1.4 1997/11/25 17:59:17 demo Exp $"
/*
 * $Log: SC2692.cc,v $
 * Revision 1.4  1997/11/25 17:59:17  demo
 *  Support setting Parity.
 *
 * Revision 1.3  1997/08/02 04:33:18  steve
 *  Support for interrupts.
 *
 * Revision 1.2  1997/04/14 05:42:14  demo
 *  Rearrange for speed and constructibility.
 *
 * Revision 1.1  1997/04/03 23:24:29  steve
 *  Add SC2692 device class.
 *
 */

# include  <chips/SC2692.h>
# include  <ucr/ucr.h>
# include  <assert.h>


/*
 * This table holds the bits needed to configure a clock select for
 * any specific baud rate. The clock_select is the contents of the
 * CSR{A,B} register. The flags holds bits from the ACR or MR0
 * registers. Note that there are a few modes that are incompatible
 * between A and B.
 */
static const struct baud_entry {
      unsigned char flags;
      unsigned char clock_select;
} baud_table[64] = {
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0xbb}, // 9600

      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0, 0},
      {0, 0}, {0, 0}, {0, 0}, {0x80, 0xcc} // 19.2K

};
#define BRG_SET_MASK 0x80

SC2692::~SC2692()
{
	// Disable channel A
      write_reg_(CRA, 0x0a);
}

void SC2692::init(volatile void*b, unsigned w)
{
      base_ = b;
      reg_width_ = w;

	// Set up the chip to use an external crystal clock.
      write_reg_(ACR, 0x30);
      write_reg_(OPCR, 0xff);

	// enable channel A, 8bits, parity none, 1 stop bits. Also
	// configure the receive interrupt to trigger (when enabled)
	// on a single character in the fifo.
      write_reg_(CRA, 0x15);
      write_reg_(MRA, 0x13);
      write_reg_(MRA, 0x07);
}

void SC2692::set_baud_a(unsigned baud)
{
      unsigned baud_idx = baud / 300 - 1;
      const struct baud_entry&entry = baud_table[baud_idx];
      write_reg_(CSRA, entry.clock_select);
      write_reg_(ACR,  entry.flags&BRG_SET_MASK | 0x30);
}

void SC2692::set_parity_a(PARITY p)
{
      unsigned char bits = 0x03;
      switch (p) {
	  case NONE:
	    bits |= 0x10;
	    break;
	  case MARK:
	    bits |= 0x0c;
	    break;
	  case SPACE:
	    bits |= 0x08;
	    break;
	  case EVEN:
	    bits |= 0x00;
	    break;
	  case ODD:
	    bits |= 0x04;
	    break;
      }
      write_reg_(CRA, 0x10);
      write_reg_(MRA, bits);
}

void SC2692::transmit_a(char value)
{
	// Wait for the transmit channel to become ready. There need
	// only be a single space in the FIFO.
      while (0 == (read_reg_(SRA) & 0x04))
	      thread_yield(); /* busy wait */

      write_reg_(TxFIFOA, value);
}

volatile unsigned char tmp;

char SC2692::receive_a()
{
	// Wait for at least one character to be ready in the input
	// FIFO.
      while (0 == (read_reg_(SRA) & 1))
	    thread_yield(); /* busy wait */

      return (char)read_reg_(RxFIFOA);
}

void SC2692::int_RxReady_a(bool flag)
{
      write_reg_(IMR, flag? 0x2 : 0);
}

void SC2692::diag_local_loop(bool flag)
{
      if (flag) {
	    write_reg_(MRA, 0x88);
	      //write_reg_(CRA, 0x05);
      } else {
	      //write_reg_(CRA, 0x0a);
	    write_reg_(MRA, 0x08);
      }
}
