/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: XC4000.cc,v 1.3 1997/04/14 05:43:11 demo Exp $"
/*
 * $Log: XC4000.cc,v $
 * Revision 1.3  1997/04/14 05:43:11  demo
 *  Update init to support rev 3 ISE boards.
 *
 * Revision 1.2  1997/03/03 22:07:32  steve
 *  Add the IMC program to the source.
 *
 * Revision 1.1  1996/11/05 00:53:32  steve
 *  Add an XC40xx programmer driver.
 *
 */

# include  <chips/XC4000.h>
# include  <chips/LED.h>
# include  <assert.h>
# include  <stddef.h>

/*
 * The code to configure works for the slave-serial mode of the XC4000
 * serial series. It works by wiggling the necessary bits of the
 * device to clock the data in, one bit at a time.
 */

XC4000::XC4000(volatile unsigned long*reg,
	       unsigned din, unsigned cclk, unsigned long m)
: reg_(reg), din_(din), cclk_(cclk), mask_(m)
{
}

XC4000::~XC4000()
{
}

/*
 * This method assumes that the magic number (0xff 0x2x) starts on a
 * byte boundry and the 24bit image length follows immediately.
 */

void XC4000::load_config(const void*dp, size_t ndata)
{
      const unsigned char*data = (const unsigned char*)dp;
      const unsigned char*end_image = data + ndata;

      LED<unsigned long>cbits(reg_, mask_);
      cbits.set(din_);
      cbits.set(cclk_);

      while (data < end_image) {
	    unsigned byte = *data;
	    for (unsigned idx = 0 ;  idx < 8 ;  idx += 1) {
		  int bit = byte & 0x80;
		  byte <<= 1;

		    // Load the bit into the DIN pin of the device, ...
		  if (bit) cbits.set(din_); else cbits.clr(din_);

		    // and clock that bit in.
		  cbits.clr(cclk_);
		  cbits.set(cclk_);
	    }

	    data += 1;
      }

	// The Xilinx takes two extra clock cycles to turn the
	// pipeline crank and get the last bit through. These may be
	// extra, but that doesn't matter.
      cbits.set(din_);
      for (unsigned idx = 0 ;  idx < 32 ;  idx += 1) {
	    cbits.clr(cclk_);
	    cbits.set(cclk_);
      }
}
