/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: bus_stdio.cc,v 1.3 1997/08/02 04:32:52 steve Exp $"
/*
 * $Log: bus_stdio.cc,v $
 * Revision 1.3  1997/08/02 04:32:52  steve
 *  Handle multiple opens on a channel.
 *
 * Revision 1.2  1997/04/21 04:48:47  steve
 *  Support partial channel reads.
 *
 * Revision 1.1  1997/02/21 21:22:15  steve
 *  Incorporate stdio interfacce to BUS cnannels.
 *
 */

# include  <chips/bus_stdio.h>
# include  <chips/BUS.h>
# include  <stdlib.h>

# define dynamic_cast reinterpret_cast

class ChannelDevice  : public StdioDevice {

    public:
      ChannelDevice(BUS*bus, unsigned id)
      : ref_count_(1), next_(0), port_(id)
      { flags_ = READABLE|WRITEABLE; bus->open_channel(&port_); }

      int read(void*str, size_t cnt)
      {
	    return port_.read(str, cnt, true);
      }

      void write(const void*str, size_t cnt)
      {
	    port_.write(str, cnt);
      }


      unsigned ref_count_;
      ChannelDevice*next_;
      BUS::Channel port_;
};

ChannelDeviceType::~ChannelDeviceType()
{
      while (list_) {
	    ChannelDevice*cur = list_;
	    list_ = cur->next_;
	    delete cur;
      }
}


StdioDevice*ChannelDeviceType::open(const char*name)
{
      const unsigned channel = strtoul(name, 0, 0);
      ChannelDevice*cur;

      for (cur = list_ ;  cur ;  cur = cur->next_)

	    if (cur->port_.channel_id() == channel) {
		  cur->ref_count_ += 1;
		  return cur;
	    }

      cur = new ChannelDevice(bus_, channel);
      cur->next_ = list_;
      list_ = cur;
      return cur;
}

void ChannelDeviceType::close(StdioDevice*dev)
{
      ChannelDevice*cur = dynamic_cast<ChannelDevice*>(dev);
      cur->ref_count_ -= 1;

      if (cur->ref_count_ == 0) {
	    if (list_ == cur) {
		  list_ = cur->next_;
	    } else {
		  ChannelDevice*idx = list_;
		  while (idx->next_ != cur) idx = idx->next_;
		  idx->next_ = cur->next_;
	    }
      }

      delete cur;
}
