/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ParallelPort_misc.cc,v 1.2 1996/10/28 03:45:53 steve Exp $"
/*
 * $Log: ParallelPort_misc.cc,v $
 * Revision 1.2  1996/10/28 03:45:53  steve
 *  License notice.
 *
 * Revision 1.1  1995/11/02 06:57:20  steve
 *  Split up the ParallelPort source,
 *  Support nibble mode id strings.
 *
 */


# include  <mc68322/ParallelPort.h>
# include  "ParallelPort_priv.h"
# include  <ucr/ucr.h>
# include  <assert.h>


/*
 * Start a DMA transfer to read as many bytes as I can. If there is a
 * read in progress, limit the transfer to what is needed.
 */
void ParallelPort::dma_go(size_t amount)
{
      transfer_  = amount;
      readp_     = buf_;
      assert( dma_ready_ );
      dma_ready_ = false;
      pdma_regs_->transfer_address = buf_;
      pdma_regs_->transfer_count = transfer_;
}

/*
 * Move bytes from the buffer ring to the specified buffer, and adjust
 * any pointers to account.
 *
 * NOTE: this assumes that holding_ is big enough.
 */
void ParallelPort::copyout(unsigned char*to, size_t count)
{
      memcpy(to, readp_, count);
      readp_   += count;
      holding_ -= count;
}


/*
 * Generic isr handler. All pdma operations can be handled here. Port
 * manipulation interrupts are more complicated so leave them to the
 * method pointed to by port_isr_.
 */
void ParallelPort::isr_(ParallelPort*obj, unsigned)
{
      const unsigned short p_flags = obj->port_intr_->intr_flags;
      const unsigned short d_flags = obj->pdma_intr_->intr_flags;
      obj->port_intr_->intr_flags = p_flags & 0x7ff;
      obj->pdma_intr_->intr_flags = d_flags & 0x007;

      if (p_flags) (obj->*obj->port_isr_)();

      assert( 0 == (d_flags & ILLEGAL_ADDRESS) );

      if ( d_flags & (DMA_COMPLETE) ) {

	      // Update parameters from completion of this transfer
	    size_t count = obj->transfer_ - obj->pdma_regs_->transfer_count;
	    obj->holding_ += count;
	    obj->transfer_  = 0;
	    obj->dma_ready_ = true;
	    obj->in_sync_.wakeup();
      }

}
