#ifndef __stdiox_H
#define __stdiox_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: stdiox.h,v 1.5 1996/10/28 03:46:06 steve Exp $"
/*
 * $Log: stdiox.h,v $
 * Revision 1.5  1996/10/28 03:46:06  steve
 *  License notice.
 *
 * Revision 1.4  1996/09/19 01:38:35  steve
 *  Add fgetc and fgets. stdio input works.
 *
 * Revision 1.3  1996/09/16 19:34:31  steve
 *  Support sprintf.
 *
 * Revision 1.2  1996/09/16 17:03:09  steve
 *  Add the read/write state machine.
 *
 * Revision 1.1  1996/09/14 00:10:22  steve
 *  Add a stdio library.
 *
 */

# include  <stdio.h>
# include  <StdioDevice.h>
# include  <stdarg.h>

struct _FILE {

	// These refer to the device that has been opened.
      StdioDeviceType *type;
      StdioDevice *dev;

      enum {
	    F_READ = 0x01,
	    F_WRITE= 0x02
      };

      unsigned flags;

	// The stdio stream has state, and can only do certain things
	// at certain times. The state limits what operations it can
	// perform next.

      enum STATE { S_IDLE, S_ERROR, S_READ, S_WRITE };
      STATE state;

	// If the stream is buffered, this is a pointer to the buffer
	// to be used. If not, it points to null.

      char*buf;

      int buf_mode;
      ptrdiff_t buf_len;
      char*buf_ptr;
      char*buf_end; // Used by read, which may get partial buffers.

      bool buf_freeme; // True if I allocated (and should free) buf

      void write_if_full()
      {
	    if ( (buf_ptr-buf) == buf_len ) {
		  dev->write(buf, buf_len);
		  buf_ptr = buf;
	    }
      }

	// Put some bytes into the buffer. If I need to write some
	// bytes to the device to make room, do so. By the time this
	// finishes, the buffer is not full.

      void write_to_buf(char ch)
      {
	    *buf_ptr++ = ch;
	    write_if_full();
      }

      void write_to_buf(const void*ptr, size_t len)
      {
	    ptrdiff_t fill = buf_ptr - buf;
	    while (len > 0) {
		  ptrdiff_t tcount = len;
		  if (fill+tcount > buf_len)
			tcount = buf_len - fill;

		  memcpy(buf_ptr, ptr, tcount);
		  buf_ptr += tcount;
		  fill += tcount;
		  ptr  += tcount;
		  len  -= tcount;

		  if (fill == buf_len) {
			dev->write(buf, buf_len);
			buf_ptr = buf;
			fill = 0;
		  }
	    }
      }
};


extern int _vformat(int (*putfun)(int, FILE*), FILE*str,
		    const char*format, va_list arg);

#endif
