
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: Ticker.cc,v 1.7 1998/01/01 07:33:56 picturel Exp $"
/*
 * $Log: Ticker.cc,v $
 * Revision 1.7  1998/01/01 07:33:56  picturel
 *  Assure initialization of table.
 *
 * Revision 1.6  1996/10/28 03:46:12  steve
 *  License notice.
 *
 * Revision 1.5  1995/10/20 04:02:41  steve
 *  Make timeouts lighter, less buggy, and a
 *  little less general. Light is better here.
 *  Also, leave lists as a general tool.
 *
 * Revision 1.4  1995/10/05 07:09:03  steve
 *  Slightly more robust ticker checking.
 *
 * Revision 1.3  1995/09/29 17:32:31  steve
 *  Define NS resolution of ticker.
 *
 * Revision 1.2  1995/09/17 07:00:30  steve
 *  Add support for timeouts.
 *
 * Revision 1.1  1995/09/02 18:27:23  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

# include  <ucr/Ticker.h>
# include  <ucr/ucr.h>
# include  <stddef.h>
# include  <assert.h>

Ticker::Ticker(unsigned long ns)
: ns_per_tick_(ns)
{
      for (unsigned idx = 0 ;  idx < TICKER_FUN_LIMIT ; idx += 1)
	fun_[idx] = 0;

      ncallout_ = 0;
      for (unsigned idx = 0 ;  idx < CALLOUT_LIMIT ; idx += 1)
	callout_[idx].fun = 0;
}

/*
 * Believe it or not, this is safe. I need not worry about being
 * interrupted when doing this, as long as I get the assignment done
 * in the correct order.
 */
void Ticker::set_tick_handler( void(*fun)(void*), void*arg )
{
      for (unsigned idx = 0 ;  idx < TICKER_FUN_LIMIT ;  idx += 1)
	if (! fun_[idx]) {
	      arg_[idx] = arg;
	      fun_[idx] = fun;
	      break;
	}
}

void Ticker::handle_tick()
{
        // Do regular tick processing
      for (unsigned idx = 0 ;  idx < TICKER_FUN_LIMIT ;  idx += 1)
	if (fun_[idx]) fun_[idx](arg_[idx]);

      unsigned idx = 0;
      for (TimeoutCell*cell = callout_; idx < ncallout_; cell += 1)
	if (cell->fun)
	  if (cell->ticks == 1) {
		void(*tmp)(void*) = cell->fun;
		cell->fun = 0;
		ncallout_ -= 1;
		tmp(cell->arg);
	  } else {
		cell->ticks -= 1;
		idx += 1;
	  }


}


Ticker::~Ticker() { }
