
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: Ticker_timeout1.cc,v 1.5 1996/10/28 03:46:12 steve Exp $"
/*
 * $Log: Ticker_timeout1.cc,v $
 * Revision 1.5  1996/10/28 03:46:12  steve
 *  License notice.
 *
 * Revision 1.4  1996/08/24 20:57:32  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.3  1995/10/20 04:02:42  steve
 *  Make timeouts lighter, less buggy, and a
 *  little less general. Light is better here.
 *  Also, leave lists as a general tool.
 *
 * Revision 1.2  1995/10/05 07:09:03  steve
 *  Slightly more robust ticker checking.
 *
 * Revision 1.1  1995/09/17 07:00:31  steve
 *  Add support for timeouts.
 *
 * Revision 1.1  1995/09/02 18:27:23  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

# include  <ucr/Ticker.h>
# include  <ucr/ucr.h>
# include  <assert.h>


/*
 * The tick handler looks in the callout table for the timeout
 * events. An active entry is marked by a non-nil function pointer,
 * and that is how I will find an inactive entry. Make sure the
 * ncallout_ member is correct, as the handler uses that count to
 * limit its scan for functions.
 */
void Ticker::timeout(unsigned ticks, void(*fun)(void*), void*arg)
{
      assert( ticks > 0 );
      assert( ncallout_ < CALLOUT_LIMIT );

      TimeoutCell*cell = callout_;
      while (cell->fun) cell += 1;

      spl_cookie_t cookie = thread_splhi();
      cell->ticks = ticks;
      cell->arg = arg;
      cell->fun = fun;
      ncallout_ += 1;
      thread_spl(cookie);
}
