/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
 .ident "$Id: locore-i960_ise.S,v 1.5 1998/05/26 16:03:36 steve Exp $"
	.file "locore.s"

 # A NOTE ON INTERRUPTS:
 # The ucr_isr-i960 file (and the uCR_isr_common routine) assume that
 # the i960 is configured for priorities 3-15. So, I do so.

 # FAULTS generate a system call. The fault handler
	.set FAULT_SYSTEM, 259

 # These are various memory mapped registers that I use.
	.set PIRSR,  0x1050
	.set MBCR,   0x1500
	.set MBBAR0, 0x1504
	.set MBBAR1, 0x1510

	.set DBCR, 0x151c
	.set DBAR, 0x1520
	.set DRWS, 0x1524
	.set DWWS, 0x1528
	.set DRIR, 0x152c

	.set IPND, 0xff008500
	.set IMSK, 0xff008504
	
 # Enable both memory banks, set bank1 to 512K, 8bit wide, and slow,
 # and set bank1 to 64K, 32bit wide, and fast.
	
	.set SET_MBCR,   0x00030033
	
	# .set SET_MBBAR0, 0xfef80000 # get this from the linker model
	.set SET_MBRWS0, 0x00070707
	.set SET_MBWWS0, 0x00070707
	
	# .set SET_MBBAR1, 0xb0000000 # get this from the linker model
	.set SET_MBRWS1, 0x00000000
	.set SET_MBWWS1, 0x00000000

 # Enable the DRAM, set all drives to normal, 1 band of EDO DRAM,
 # 4Meg per leaf. Put the DRAM in region 2_3

	.set SET_DBCR, 0x00000043
	# .set SET_DBAR, 0xa0000000 # get this from the linker model
	.set SET_DRWS, 0x00000000
	.set SET_DWWS, 0x00000000
	.set SET_DRIR, 0x000001F0

 # Allocate an interrupt stack (local only). The supervisor stack is
 # also the main stack and is allocated elsewhere.

	.bss	isr_stack, 2048, 6

	.text
	.global start
start:

			# Set up DRAM parameters
	lda	__ISE_DRAM, r4
	ldconst SET_DRWS, r5
	ldconst SET_DWWS, r6
	ldconst SET_DRIR, r7
	stq	r4, DBAR

			# and enable them.
	ldconst	SET_DBCR, r4
	st	r4, DBCR

			# Set up the FLASH memory parameters...
	lda	__ISE_FLASH, r4
	ldconst SET_MBRWS0, r5
	ldconst SET_MBWWS0, r6
	stt	r4, MBBAR0

			# Set up the ISE_STATUS/CONTROL memory parameters...
	lda	__ISE_STATUS, r4
	ldconst SET_MBRWS1, r5
	ldconst SET_MBWWS1, r6
	stt	r4, MBBAR1

			# and activate them.
	ldconst SET_MBCR, r4
	st	r4, MBCR

			# Send S_INTx# to the core processor through the
			# XINT0:3 signals, and NOT through the primary IRQs.
	mov	0xf, r4
	st	r4, PIRSR

			# Switch onto the supervisor stack.
			# (NOTE: No registers will survive this.)
	call	super_stack
start_cont:

			# Clear pending interrupts and unmask them.
	mov	0, r4
	st	r4, IPND
	
	ldconst	0x30ff, r4
	st	r4, IMSK
	inten

			# Write the __index_space__ address into the
			# pci to local address translation register.
	lda	__index_space__, r4
	st	r4, 0x1244

 # Here we GO! The GCC calling sequence uses g14 as an argument pointer,
 # which is set to 0 if there are no extension arguments. This MUST start
 # out as 0, because GCC is quite smart enough to use it as a magic source
 # of zeros when a 0 in a register is required.

	mov	0, g14
	call	_ucr_start

	halt 0


 # This nasty bit of magic switches the CPU off the interrupt stack and
 # onto the supervisor stack. It also flips off the PC.s bit. It works
 # by building up a stack frame on the desired stack, and switching the
 # pfp to point to it. I then bum up a PC and AC on the current, soon to
 # be abandoned, stack and do a Return From Interrupt. This turns off the
 # PC.s bit, and initializes the PC how I want it.

super_stack:
			# Make a new stack frame...
	mov	0, r4          # PFP
	lda	_bstack+64, r5 # SP
	lda	start_cont, r6 # RIP
	stt	r4, _bstack

			# Point into the new stack frame...
	lda	_bstack, r4
	flushreg
	or	r4, 0x07, pfp
	flushreg

			# Munge up a new PC and AC register...
	ldconst	0x001f0003, r8
	ldconst 0x00000000, r9
	stl	r8, -16(fp)

	ret # Whew!

nmi_fake:
	lda	nmi_string, g0
	mov	0, g1
	mov	0, g14
	call	_ucr_panic
nmi_string:
	.asciz	"nmi detected"


 # The PRCB points to all the various tables that the CPU uses in its
 # cycle-to-cycle business. The IBR points to this.

	.balign	4
	.global prcb
prcb:
	.word	fault_table
	.word	control_table
	.word	0		# AC Register Initial Image
	.word	0x40000000	# Fault configuration word
	.word	isr_table
	.word	sys_proc
	.word	0
	.word	isr_stack
	.word	0x00000000	# enable instruction cache
	.word	0		# Allow full register caching


 # All faults are handled in a common system call. The call uses the
 # fault-type of the fault record to decide what to do.

	.balign	4
fault_table:
	.rept	32
	.word	FAULT_SYSTEM*4+2
	.word	0x027f
	.endr


	.balign	4
control_table:
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0x00009876 # IMAP0
	.word	0x0000dcba # IMAP1
	.word	0x00fe0000 # IMAP2 (map the 2 timer interupts)
	.word	0x00004800 # ICON (start with interrupts disabled)
	
	.word	0x00800000 # PMCON0_1 (CPU on-chip memory/devices)
	.word	0
	.word	0x00800000 # PMCON2_3
	.word	0
	.word	0x00800000 # PMCON4_5
	.word	0
	.word	0x00800000 # PMCON6_7
	.word	0
	.word	0x00800000 # PMCON8_9 (Outbound translation windows)
	.word	0
	.word	0x00800000 # PMCON10_11 (DRAM and ISE I/O here)
	.word	0
	.word	0x00800000 # PMCON12_13
	.word	0
	.word	0x00000000 # PMCON14_15
	.word	0

	.word	0
	.word	0
	.word	0 # Initial trace-controls register
	.word	0x01 # BCON (enable PMCON registers, leave low RAM writeable)


 # Currently, the only use of the system procedure table is to call
 # the fault handler. The fault table uses number 259 as the fault code.
	.balign	4
sys_proc:
	.word	0
	.word	0
	.word	0
	.word	_bstack+0 # (Supervisor mode stack, and Trace Control bit)
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.rept	259
	.word	0
	.endr
	.word	_uCR_fault_common+2 # (sys_call 259 is fault action)


 # The isr_table is all the interrupt vectors for the CPU. These go
 # in initialized RAM, because the CPU scribbles of part of the table.

	.sect	.data
	.balign	4
isr_table:
	.word	0
	.rept 3
	.word	0, 0, 0, 0, 0, 0, 0, 0
	.word	0, 0, 0, 0, 0, 0, 0, 0
	.endr

	.rept 13
	.word	nmi_fake, 0, _uCR_isr_common, 0, 0, 0, 0, 0
	.word	nmi_fake, 0, 0, 0, 0, 0, 0, 0
	.endr

 #
 # The Initial Boot Record is read by the CPU at reset and is used to
 # find the prcb and the start address. This must be placed is a specific
 # place in memory (which the linker does) so the specific linkage
 # requirements justify placing this in a special section. There is no
 # need for a symbol to start it, because it is the only thing in the
 # rather specialized section.
 #
 # NOTE: The ibr_checksum is calculated for us by the linker.

	.sect ibr
	.word	0
	.word	0
	.word	0
	.word	0
	.word	start
	.word	prcb
	.word	-2
	.word	0xaaaaaaaa
	.word	0x55555555
	.word	0xaaaaaaaa
	.word	0x55555555
	.word	ibr_checksum

/*
 * $Log: locore-i960_ise.S,v $
 * Revision 1.5  1998/05/26 16:03:36  steve
 *  Add panic.
 *
 * Revision 1.4  1998/04/03 01:35:12  steve
 *  Add assertion support.
 *
 * Revision 1.3  1997/12/31 01:24:04  steve
 *  Make flash writeable.
 *
 * Revision 1.2  1997/08/26 19:25:49  steve
 *  cleanup.
 *
 * Revision 1.1  1997/08/25 17:43:02  steve
 *  Allow memory layout be set by link models.
 */
