
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: set_isr.cc,v 1.9 1996/10/28 03:46:34 steve Exp $"
/*
 * $Log: set_isr.cc,v $
 * Revision 1.9  1996/10/28 03:46:34  steve
 *  License notice.
 *
 * Revision 1.8  1996/08/24 20:57:34  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.7  1996/08/10 04:00:56  steve
 *  Handle limited isr optimally.
 *
 * Revision 1.6  1996/08/08 05:22:44  steve
 *  Get rid of the __Scheduler class, it was not
 *  needed. Add the thread_runnable and _reschedule
 *  functions for thread scheduling.
 *
 * Revision 1.5  1996/08/03 03:30:42  steve
 *  Create the ucr/asm.h header as a target specific
 *  header with line functions, etc; Replace the
 *  __isr_run_flag with a bit of inline assembly code.
 *
 *  Move much thread code out of the __Secheduler class
 *  and into more public thread_ functions, to eliminate
 *  some wasted function call depth. Even the main_thread
 *  is move into its own file.
 *
 *  Keep a list of available threads for debug use, and
 *  add a function to get thread registers.
 *
 * Revision 1.4  1996/08/02 16:54:17  steve
 *  Fix assertion of vec number.
 *
 * Revision 1.3  1996/08/02 00:17:59  steve
 *  Move the param.h header file.
 *
 * Revision 1.2  1996/07/19 23:47:37  steve
 *  Calling thread_pause in an isr is not OK. Also,
 *  make thread switches in ISRs OK. The assembly
 *  code hides the fact that that switch actually
 *  takes affect when the ISR returns.
 *
 * Revision 1.1  1996/05/29 04:36:06  steve
 *  Support interrupts in the i960_cyclone world, and in the
 *  process reorganize interrupt code for portability.
 */

# include  <ucr/ucr.h>
# include  <ucr/param.h>
# include  <assert.h>


struct soft_vector {
      void (*fun)(void*, unsigned);
      void* arg;
};

static soft_vector isr_table[ MAX_EXCEPTION*MAX_ISR_PER_EXCEPTION ] = { {0} };

/*
 * There is architecture-specific code that arranges for this function
 * to be called with the correct parameter. This routine then arranges
 * for the registered interrupt handlers to be called with the right
 * parameters.
 */
extern "C" void uCR_isr_base(unsigned long intr)
{
      soft_vector*cur = isr_table + intr * MAX_ISR_PER_EXCEPTION;

#if MAX_ISR_PER_EXCEPTION > 1
      for (unsigned idx = 0
	   ;  idx < MAX_ISR_PER_EXCEPTION
	   ;  idx += 1, cur += 1)
	if (cur->fun) (cur->fun) (cur->arg, intr);
#else
      if (cur->fun) (cur->fun) (cur->arg, intr);
#endif
}

/*
 * Place the function and arg in the table of routines. Do it
 * atomically (that is, protected from interrupts) because, after all,
 * an interrupt will look in this table to get the isr.
 */
void set_isr(unsigned vec, void (*fun)(void*, unsigned), void*arg)
{
      assert( vec < MAX_EXCEPTION );
      spl_cookie_t cookie = thread_splhi();

#if MAX_ISR_PER_EXCEPTION == 1
      assert( 0 == isr_table[vec].fun );
      isr_table[vec].fun = fun;
      isr_table[vec].arg = arg;
#else
      vec *= MAX_ISR_PER_EXCEPTION;

      unsigned idx = 0;
      for (idx = 0 ;  idx < MAX_ISR_PER_EXCEPTION ;  idx += 1)

	if (! isr_table[vec+idx].fun) {
	      isr_table[vec+idx].fun = fun;
	      isr_table[vec+idx].arg = arg;
	      break;
	}

      assert( idx < MAX_ISR_PER_EXCEPTION );
#endif


      thread_spl(cookie);
}
