
#
# Copyright (c) 1996 Picture Elements Inc.
#	Stephen Williams (steve@picturel.com)
#
#    This source code is free software; you can redistribute it
#    and/or modify it in source code form under the terms of the GNU
#    Library General Public License as published by the Free Software
#    Foundation; either version 2 of the License, or (at your option)
#    any later version. In order to redistribute the software in
#    binary form, you will need a Picture Elements Binary Software
#    License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this program; if not, write to the Free
#    Software Foundation, Inc.,
#    59 Temple Place - Suite 330
#    Boston, MA 02111-1307, USA
#
#    You should also have recieved a copy of the Picture Elements
#    Binary Software License offer along with the source. This offer
#    allows you to obtain the right to redistribute the software in
#    binary (compiled) form. If you have not received it, contact
#    Picture Elements, Inc.,
#    777 Panoramic Way
#    Berkeley, CA 94704.
#
.ident "$Id: setjmp-i960.s,v 1.1 1996/10/29 01:31:32 steve Exp $"
#
# $Log: setjmp-i960.s,v $
# Revision 1.1  1996/10/29 01:31:32  steve
#  Implement the setjmp and longjmp routines
#  for the i960 and add them in the documentation.
#

	.file "setjmp-i960.s"
	.text

# This function stores the necessary context in the jmp_buf and returns
# a 0. The context is enough for longjmp to cause a return to the point
# where this was called, with a return code other then 0.

# The words of the jmp_buf are allocated like so:
#	env[0-6]  - g8 - g14
#	env[7]    - pfp
#	env[8-24] - r0-r15
# The local registers are save on the stack frame automatically by the
# i960. All the other registers are not available across function calls
# so need not be saved.


# extern int setjmp(jmp_buf env)

	.global _setjmp
_setjmp:

	stq	g8, 0(g0)
	stt	g12, 16(g0)
	st	pfp, 28(g0)

 # Copy the local registers from the PFP into the save area. I'll use these
 # to restore the PF of the setjmp when the longjmp happens. Use memcpy!

	lda	32(g0), g0

	flushreg
	ldq	0(pfp), r4
	stq	r4, 0(g0)
	ldq	16(pfp), r4
	stq	r4, 16(g0)
	ldq	32(pfp), r4
	stq	r4, 32(g0)
	ldq	48(pfp), r4
	stq	r4, 48(g0)

	mov	0, g0
	ret

# The longjmp function takes an environment stored by a previous setjmp
# and uses it to restore the register context that the setjmp came from.
# By also restoring the pfp, the longjmp also restores the stack position
# and rip. This causes a ret from here to appear as a return from the
# setjmp. (That is why the longjmp can never return.)

# extern void longjmp(jmp_buf env, int val) __attribute__((noreturn))

	.global	_longjmp
_longjmp:

 # Restore the global registers

	ldq	0(g0), g8
	ldt	16(g0), g12

 # Copy the local registers of interest into the previous frame. This
 # copy happens in the SAME stack (longjmp is only allowed to pop frames)
 # so be careful where to do flushreg.

	flushreg
	ld	28(g0), r8
	ldq	32+0(g0), r4
	stq	r4, 0(r8)
	ldq	32+16(g0), r4
	stq	r4, 16(r8)
	ldq	32+32(g0), r4
	stq	r4, 32(r8)
	ldq	32+48(g0), r4
	stq	r4, 48(r8)

 # Make sure the return code to setjmp is not zero. Change it if I must.

	mov	g1, g0
	cmpobne	0, g0, 0f
	mov	1, g0
0:
	flushreg
	mov	r8, pfp
	flushreg
	ret
