#ifndef __thread_cookie_H
#define __thread_cookie_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: thread_cookie-i960.h,v 1.11 1997/08/25 21:24:22 steve Exp $"
/*
 * $Log: thread_cookie-i960.h,v $
 * Revision 1.11  1997/08/25 21:24:22  steve
 *  Add some stack thread checks.
 *
 * Revision 1.10  1996/10/28 03:46:39  steve
 *  License notice.
 *
 * Revision 1.9  1996/08/24 20:57:35  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.8  1996/08/22 00:36:10  steve
 *  the stack pointer is not handled seperately.
 *
 * Revision 1.7  1996/08/18 22:36:24  steve
 *  Switch hte TC register with the thread.
 *
 * Revision 1.6  1996/07/22 17:57:58  steve
 *  Enable tracing at non-interrupt level,
 *  make the locore startup code allocate the stack
 *  frame to be used.
 *
 * Revision 1.5  1996/07/19 23:45:04  steve
 *  Start threads with supervisor bit set.
 *
 * Revision 1.4  1996/07/19 04:29:24  steve
 *  change ISR handling to do thread switching when the return
 *  from interrupt happens.
 *
 * Revision 1.3  1996/05/29 22:52:52  steve
 *  Simplify and optimize i960 thread switching.
 *
 * Revision 1.2  1996/05/29 22:07:24  steve
 *  Properly switch pc register with threads.
 *
 * Revision 1.1  1996/05/27 22:50:01  steve
 *  implement thread_create and thread_switch for the i960,
 *  rename some __foo symbols to uCR_foo for neatness.
 *
 */

# define REGISTER_COUNT 16

/*
 * These are predefined and interesting offsets into the stack
 * frame. These are defined by the i960 calling sequence.
 */
# define FRAME_PFP 0
# define FRAME_SP  1
# define FRAME_RIP 2

/*
 * The registers are stored in the regs array. Only the global
 * registers and processor status registers are saved. The local
 * registers are automatically pushed by the call sequence. Even if
 * the interrupt handler calls the stack switch, the call to the
 * switch is sufficient to save registers.
 */

/*
 * When a thread is suspended by a uCR_switch, the frame context is
 * saved as follows (frame #1 is where uCR_thread is called from):
 *
 *			    |    |
 *	        (16 words)  |    |
 *			    |    |
 *   frame #0 (uCR_thread)  +----+
 *			    |    |
 *			    |    |
 *   frame #1		    +----+  <-- cookie.regs[15] (fp)
 *
 * When frame #0 returns, cookie.regs[15] is loaded to pfp. Then the
 * return causes it to be moved to fp, and the stack pointer to be loaded
 * from the previous frame. Therefore, the fp and sp of the current thread
 * are meaningless and do not need to be restored in a thread switch.
 *
 * When an interrupt causes the thread to be saved, frame #0 in the
 * picture is uCR_isr_common and frame #1 is the interrupted frame.
 *
 * In either case, the RIP for the return is in (cookie.regs[15])[2] and so
 * lives in the thread's stack.
 */

struct ThreadCookie {
      unsigned long regs [REGISTER_COUNT];
      unsigned long pc;
      unsigned long ac;
      unsigned long tc;
};

# define COOKIE_INITIALIZER { { 0 }, 3, 0, 0 }

static inline void thread_cookie_init(ThreadCookie*cookie)
{
      for (unsigned idx = 0 ;  idx < REGISTER_COUNT ;  idx += 1)
	    cookie->regs[idx] = 0;

      cookie->pc = 0x00000003;
      cookie->ac = 0;
      cookie->tc = 0;
}

# define CPU_STACK_REGISTER "sp"

#endif
