/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: thread_runnable.cc,v 1.6 1997/08/26 19:27:05 steve Exp $"
/*
 * $Log: thread_runnable.cc,v $
 * Revision 1.6  1997/08/26 19:27:05  steve
 *  Stack checks cannot occur everywhere because the
 *  ISR stack may be outside the stack proper.
 *
 * Revision 1.5  1997/08/25 21:24:22  steve
 *  Add some stack thread checks.
 *
 * Revision 1.4  1996/10/28 03:46:42  steve
 *  License notice.
 *
 * Revision 1.3  1996/08/24 21:20:10  steve
 *  Optimize thread_yield quite a bit.
 *
 * Revision 1.2  1996/08/24 20:57:37  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.1  1996/08/08 05:22:46  steve
 *  Get rid of the __Scheduler class, it was not
 *  needed. Add the thread_runnable and _reschedule
 *  functions for thread scheduling.
 *
 */

# include  <ucr/ucr.h>
# include  <ucr/ThreadQueue.h>
# include  "__thread.h"
# include  <assert.h>

extern THREAD*uCR_current_thread;
static ThreadQueue runq_ (uCR_current_thread);

void thread_runnable(THREAD*tid, bool front_flag)
{
      spl_cookie_t splcookie = thread_splhi();
      runq_.enqueue(tid, front_flag);
      thread_spl(splcookie);
}

void thread_reschedule()
{
      THREAD*thr;

      spl_cookie_t cookie = thread_splhi();

      while ( (thr = runq_.peek()) == 0 ) {
	    if (isr_hot_flag()) break;
	    assert( ! uCR_thread_stack_check() );
	    thread_pause();
      }

      if (thr && (thr != uCR_current_thread)) thr_switch_(thr);
      thread_spl(cookie);
}

void thread_yield()
{
      spl_cookie_t cookie = thread_splhi();
      runq_.enqueue(uCR_current_thread, false);
      THREAD*thr = runq_.peek();
      if (thr != uCR_current_thread) thr_switch_(thr);
      thread_spl(cookie);
}
