/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: thread_switch.cc,v 1.7 1996/10/28 03:46:44 steve Exp $"
/*
 * $Log: thread_switch.cc,v $
 * Revision 1.7  1996/10/28 03:46:44  steve
 *  License notice.
 *
 * Revision 1.6  1996/08/25 00:29:12  steve
 *  Optimize thread switching.
 *
 * Revision 1.5  1996/08/24 20:57:37  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.4  1996/08/22 00:36:11  steve
 *  the stack pointer is not handled seperately.
 *
 * Revision 1.3  1996/08/10 01:03:33  steve
 *  Remove excessive assert.
 *
 * Revision 1.2  1996/08/08 05:22:48  steve
 *  Get rid of the __Scheduler class, it was not
 *  needed. Add the thread_runnable and _reschedule
 *  functions for thread scheduling.
 *
 * Revision 1.1  1996/08/03 03:30:49  steve
 *  Create the ucr/asm.h header as a target specific
 *  header with line functions, etc; Replace the
 *  __isr_run_flag with a bit of inline assembly code.
 *
 *  Move much thread code out of the __Secheduler class
 *  and into more public thread_ functions, to eliminate
 *  some wasted function call depth. Even the main_thread
 *  is move into its own file.
 *
 *  Keep a list of available threads for debug use, and
 *  add a function to get thread registers.
 *
 */

# include  <ucr/ucr.h>
# include  "__thread.h"
# include  <assert.h>


/*
 * Switch the current thread to the specified thread.
 * A thread switch puts the target thread into the front of the run
 * queue, and the currently running thread on the end.
 */
void thread_switch(THREAD*tid)
{
      if (thread_self() == tid) return;
      spl_cookie_t splcookie = thread_splhi();


#if defined(m68k_coff)
      assert( tid->cookie.regs[REGISTER_SP] <= (unsigned long)tid->estack );
      assert( tid->cookie.regs[REGISTER_SP] >= (unsigned long)tid->bstack );
#elif defined(i960_cyclone)
	//assert( tid->cookie.regs[15] <  (unsigned long)tid->estack );
	//assert( tid->cookie.regs[15] >= (unsigned long)tid->bstack );
#endif


      thread_runnable(tid, true);
      thread_runnable(uCR_current_thread, false);
      thr_switch_(tid);
      thread_spl(splcookie);
}
