
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: uCR_stack-i960.cc,v 1.1 1996/11/08 21:40:45 steve Exp $"
/*
 * $Log: uCR_stack-i960.cc,v $
 * Revision 1.1  1996/11/08 21:40:45  steve
 *  Divide up thread_create for easier porting, Add the
 *  sparc_linux and i386_linux targets, and adjust makefiles
 *  to allow for them. Document the new concept of virtual boards.
 *
 *  Change the linker output file extension to .exe, because
 *  some targets use ELF and not COFF.
 *
 *  Get rid of some trivial warnings.
 *
 */

# include  "__thread.h"

void uCR_stack_init(THREAD*tid,
		    void (*base)(void (*)(void*), void*),
		    void (*fun)(void*), void*arg)
{
	// The context I build up here is such that when the ret is
	// called, the frame is set up to look like base_of_thread was
	// just entered with parameters (fun, arg). The previous frame
	// for the base_of_thread holds only the 16 local registers.

	// After the return, the registers should be:
	//  PFP (r0):  0
	//  SP  (r1)   tib->bstack + 16 words
	//  IP         &base_of_thread
	//  FP  (g15)  tib->bstack
	//  g0         fun
	//  g1         arg
      void**bsp = (void**) (((unsigned long)tid->bstack + 15) & ~15);
      bsp[FRAME_PFP] = 0;
      bsp[FRAME_SP]  = bsp + 16;
      bsp[FRAME_RIP] = base;

      tid->cookie.regs[0]  = (unsigned long)fun;
      tid->cookie.regs[1]  = (unsigned long)arg;
      tid->cookie.regs[15] = (unsigned long)bsp;
}
