
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: uCR_stack-sparc.cc,v 1.1 1996/11/08 21:40:46 steve Exp $"
/*
 * $Log: uCR_stack-sparc.cc,v $
 * Revision 1.1  1996/11/08 21:40:46  steve
 *  Divide up thread_create for easier porting, Add the
 *  sparc_linux and i386_linux targets, and adjust makefiles
 *  to allow for them. Document the new concept of virtual boards.
 *
 *  Change the linker output file extension to .exe, because
 *  some targets use ELF and not COFF.
 *
 *  Get rid of some trivial warnings.
 *
 */


# include  "__thread.h"

/*
 * The sparc stack frame needs to look like that of a stack that is
 * about to call *base. *base is going to first thing SAVE to create
 * its own frame. This frame is entered from the switcher which will
 * use RESTORE to pop into this frame, then branch to the first
 * instruction, just like doing a return. The idea is to be able to
 * start this thread by loading %fp to point to my stack and doing:
 *
 *	ret
 *	restore
 *
 * The sparc stack grows downward, like any well-behaved CPU
 * should. The fake frame I'm building is 16 words, just big enough to
 * hold the 8 in and 8 local registers of the frame. The registers are
 * stored in the frame in increasing address as (unsigned long*fp):
 *
 *	%i7  -- fp[15]
 *	    ...
 *	%i0  -- fp[8]
 *	%l7  -- fp[7]
 *	    ...
 *	%l0  -- fp[0]
 *
 * So what should happen here? What I want is for the registers to
 * look like this after a restore with the correct pointer and before
 * the jump into the thread:
 *
 *	%i7  -- base - 8bytes
 *	%fp  -- 0  (must be zero to mark the end of the stack chain)
 *	%i5  -- xxx
 *	%i4  -- xxx
 *	%i3  -- xxx
 *	%i2  -- xxx
 *	%i1  -- xxx
 *	%i0  -- xxx
 *
 *	%l7  -- xxx
 *	%l6  -- xxx
 *	%l5  -- xxx
 *	%l4  -- xxx
 *	%l3  -- xxx
 *	%l2  -- xxx
 *	%l1  -- xxx
 *	%l0  -- xxx
 *
 *	%o7  -- xxx
 *	%sp  -- tid->estack - 16words
 *	%o5  -- xxx
 *	%o4  -- xxx
 *	%o3  -- xxx
 *	%o2  -- xxx
 *	%o1  -- arg
 *	%o0  -- fun
 *
 * The thread switcher enters the thread with ``jmpl %i7+8,%g0'' once
 * the registers are set up so. The effect is that the first
 * instruction of the thread fuction, [base], is the first instruction
 * executed and it sees the parameters base(fun, arg). Bingo!
 */

void uCR_stack_init(THREAD*tid,
		    void (*base)(void (*)(void*), void*),
		    void (*fun)(void*), void*arg)
{
      unsigned long*esp = (unsigned long*)tid->estack;
      unsigned long*fp  = esp - 16;

	// The %i and %l registers are in the frame I'm building,
	// which starts 16 words from the top of the stack.

      fp[14] = 0;
      fp[15] = ((unsigned long)base) - 8;

	// The %o[] and %g[] registers don't go in the stack, so they must
	// come from the cookie. Sure enough, I made space for them.

      tid->cookie.regs[SPARC_REG_OUT(0)] = (unsigned long)fun;
      tid->cookie.regs[SPARC_REG_OUT(1)] = (unsigned long)arg;
      tid->cookie.regs[SPARC_REG_OUT(6)] = (unsigned long)fp; // SP
      tid->cookie.regs[SPARC_REG_OUT(7)] = fp[15];
}
