/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: main.cc,v 1.10 1997/06/16 05:14:47 steve Exp $"
/*
 * $Log: main.cc,v $
 * Revision 1.10  1997/06/16 05:14:47  steve
 *  New channel protocol design and implementation.
 *
 * Revision 1.9  1997/04/20 19:17:55  steve
 *  Reset xilinx devices on restart.
 *
 * Revision 1.8  1997/04/16 22:20:26  steve
 *  Add an ISE standalone tester.
 *
 * Revision 1.7  1997/04/14 05:38:18  demo
 *  symbol name change.
 *
 * Revision 1.6  1997/03/25 00:24:00  steve
 *  Make devices private, and load the IMC.
 *
 * Revision 1.5  1997/03/03 22:07:39  steve
 *  Add the IMC program to the source.
 *
 * Revision 1.4  1996/10/28 03:46:50  steve
 *  License notice.
 *
 * Revision 1.3  1996/10/22 03:59:19  steve
 *  Create a debug.h header for debugger tools other then
 *  the GDB class itself, and create a gdb_thread function
 *  to start a GDB thread.
 *
 * Revision 1.2  1996/10/16 00:06:51  steve
 *  Support read to nowhere.
 *
 * Revision 1.1  1996/10/08 03:28:52  steve
 *  Add the ISE bootstrap prom.
 *
 */

# include  <chips/I960RP.h>
# include  <chips/BUS.h>
# include  <chips/ise_board.h>
# include  <ucr/ucr.h>
# include  "coff.h"

static volatile unsigned short& EBCR = *(volatile unsigned short*)0x1040;
static volatile unsigned short& SISR = *(volatile unsigned short*)0x1042;

inline unsigned get_short(const unsigned char cp[2])
{
      return cp[0] + (cp[1] << 8);
}

inline unsigned get_long(const unsigned char cp[4])
{
      return cp[0] + (cp[1] << 8) + (cp[2] << 16) + (cp[3] << 24);
}

inline unsigned char* get_ptr(const unsigned char cp[4])
{
      unsigned long res = cp[0] + (cp[1] << 8) + (cp[2] << 16) + (cp[3] << 24);
      return (unsigned char*)res;
}

inline void i960_sysinit(void*prcb, void*start)
{
      unsigned type = 0x300;
      asm volatile ( "sysctl %0, %1, %2"
		     :
		     : "r" (type), "r" (start), "r" (prcb) );
}

BUS bus;

BUS::Channel chan (0);

I960RP i960rp;


/*
 * Up to 8 sections are supported in the input file. The offset is the
 * location in the input stream where the section lives, and the addr
 * is the LOAD address. As the sections come in, they are shoved
 * directly into DRAM.
 */

struct sect_info_t {
      unsigned long offset;
      unsigned char*addr;
      size_t size;

      bool ibr_flag; // Turned on only for the IBR section.
} sect_info[8];

unsigned sect_info_cnt = 0;

/*
 * The i960 has an Initial Boot Record that uCR places in a special
 * COFF section, for direct placement. The BOOTPROM simulates the IBR
 * by reading it into this little stretch mf memory and interpreting
 * the parts when resetting the CPU.
 */
struct {
      unsigned long pad00[4];
      void* start;
      void* prcb;
      unsigned long pad18[6];
} simulate_ibr;

/* ** */


main()
{
	/* Make sure the Xilinx reset lines go low for at least a
	   brief moment, so that the parts get reset. This may be a
	   bit extreme in general, but is indispensable when trying to
	   test Xilinx programs. */
      _ISE_CONTROL = 0;
      _ISE_CONTROL = 0x140;

	/* Make EVERYTHING private. The ISE board is not up to
	   anything other. */
      SISR = 0x1f;

	/* This clears the Configuration Cycle Retry bit of the
	   EBCR register. At this point, the host BIOS gains
	   access to the board. */
      EBCR = 0;

      bool ready_to_reset = false;
      unsigned long cur_offset = 0;

      bus.open_device (&i960rp);
      bus.open_channel(&chan);


      _ISE_CONTROL = 0x148;

      while (! ready_to_reset) {

	    file_header coff_head;

	    chan.read(&coff_head, sizeof coff_head);
	    cur_offset = sizeof coff_head;

	    _ISE_CONTROL = 0x14c;

	      // XXXX Here would be a good place to check for a
	      // reasonable magic number. XXXX

	      /* Read past the optional extension header. There is
		 nothing of value for us there. */
	    {
		  unsigned ext_skip = get_short(coff_head.f_opthdr);
		  chan.read(0, ext_skip);
		  cur_offset += ext_skip;
	    }

	      /* Now we are at the section headers. Read them, looking
		 for those of interest. Save pointers and information
		 because the section contents have not arrived yet. */

	    for (unsigned nsect = get_short(coff_head.f_nscns)
		       ; nsect
		       ; nsect -= 1) {

		  section_header sect;
		  chan.read(&sect, sizeof sect);
		  cur_offset += sizeof sect;

		  sect_info[sect_info_cnt].offset = get_long(sect.s_scnptr);
		  sect_info[sect_info_cnt].addr = get_ptr(sect.s_paddr);
		  sect_info[sect_info_cnt].size = get_long(sect.s_size);

		  if (strcmp(sect.s_name,"ibr") == 0)
			sect_info[sect_info_cnt].ibr_flag = true;
		  else
			sect_info[sect_info_cnt].ibr_flag = false;

		  sect_info_cnt += 1;
	    }


	      /* Sort all the section information records so that I
		 can just go through them in order while reading the
		 file in. It makes the following logic much easier. */

	    for (unsigned idx = 0 ;  idx < (sect_info_cnt-1) ;  idx += 1)
		  for (unsigned jdx = idx+1
			     ; jdx < sect_info_cnt
			     ;  jdx += 1)

			if (sect_info[jdx].offset < sect_info[idx].offset) {
			      sect_info_t tmp = sect_info[jdx];
			      sect_info[jdx] = sect_info[idx];
			      sect_info[idx] = tmp;
			}


	      /* For each supported section, read up to the section,
		 then read the section bytes into its destination in
		 memory. */

	    for (unsigned idx = 0 ;  idx < sect_info_cnt ;  idx += 1) {
		  sect_info_t&sect = sect_info[idx];

		  if (sect.offset > cur_offset) {
			chan.read(0, sect.offset - cur_offset);
			cur_offset = sect.offset;
		  }

		  if (sect.ibr_flag) {

			  // Interpret the IBR by reading the start
			  // address and the prcb pointer out of it.
			chan.read(&simulate_ibr, sizeof simulate_ibr);
			cur_offset += sizeof simulate_ibr;
			ready_to_reset = true;

		  } else {

			  // Stuff the bytes right where they belong.
			chan.read(sect.addr, sect.size);
			cur_offset += sect.size;
		  }
	    }

      }

      _ISE_CONTROL = 0x144;

      i960rp.bist_enable();
      i960rp.bist_wait();
      i960rp.bist_enable(false);
      i960rp.bist_finished(0);

      i960_sysinit(simulate_ibr.prcb, simulate_ibr.start);
}
