#ifndef __os_linux_H
#define __os_linux_H
/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: os-linux.h,v 1.9 1998/03/23 00:07:38 steve Exp $"

# include  <linux/module.h>
# include  <linux/version.h>
#if LINUX_VERSION_CODE > 0x020100
# include  <linux/modversions.h>
#endif
# include  <linux/wrapper.h>
# include  <linux/bios32.h>
# include  <linux/pci.h>
# include  <linux/mm.h>
# include  <linux/sched.h>
# include  <linux/fs.h>
# include  <linux/signal.h>
# include  <linux/errno.h>
# include  <linux/malloc.h>
# include  <linux/types.h>
# include  <asm/segment.h>
# include  <asm/io.h>
# include  <asm/pgtable.h>

struct Instance;

/* This little convenience routine returns true if the current process
   has been signalled. */

#if LINUX_VERSION_CODE < 0x020100
static int inline is_signalled(struct Instance*xsp)
{ return (current->signal & ~current->blocked) != 0; }
#else
static int inline is_signalled(struct Instance*xsp)
{ return signal_pending(current); }
#endif

#if LINUX_VERSION_CODE < 0x020100
# define copy_to_user memcpy_tofs
# define copy_from_user memcpy_fromfs
#else
# include  <asm/uaccess.h>
#endif

/* A buffer accessible to both the driver and the device may have two
   different addresses, depending on who you are. This structure holds
   both addresses for the same buffer. */
struct shared_buf {
      void*host;
      __u32 bus;
};

/* The pri is GPF_KERNEL if called from a process context, or
   GPF_ATOMIC if called in an interrupt handler. */

static inline struct shared_buf alloc_buf()
{
      struct shared_buf result;
      result.host = (void*)get_free_page(GFP_KERNEL|GFP_DMA);
      result.bus = virt_to_bus(result.host);
      return  result;
}

static inline void free_buf(struct shared_buf buf)
{ free_page((unsigned long)buf.host); }

# define PACKET_MTU PAGE_SIZE

/* Linux needs the irq member to keep track of the irq that is
   assigned to the board. It uses this to remove the device later. */
# define OS_INST_MEMBERS unsigned char irq;

/* Take the device_fn of the i960RP ATU and generate the matching
   device_fn for the bridge. This works by subtracting 1 from the
   function unit. */
#define i960rp_to_bridge(x) ((x)-1)

/*
 * $Log: os-linux.h,v $
 * Revision 1.9  1998/03/23 00:07:38  steve
 *  initial frame support for winnt.
 *
 * Revision 1.8  1998/03/21 21:06:43  steve
 *  Rearrange frame handling code.
 *
 * Revision 1.7  1998/03/18 20:40:17  steve
 *  compile on 2.0 kernels (alpha)
 *
 * Revision 1.6  1997/12/06 05:15:51  steve
 *  Add the control device to the NT port.
 *
 * Revision 1.5  1997/12/02 02:29:03  steve
 *  Add support for ISE board reset.
 *
 * Revision 1.4  1997/11/16 11:37:43  steve
 *  Make NT driver asynchronous.
 *
 * Revision 1.3  1997/11/11 11:19:45  steve
 *  Add support for frames.
 *
 * Revision 1.2  1997/09/12 05:18:06  steve
 *  Port to linux 2.1
 *
 * Revision 1.1  1997/09/11 21:58:36  steve
 *  Port the uCR device driver to Windows NT,
 *  make the configure select the target board
 *  and target host interface headers.
 *
 */


#endif
