#ifndef __t_c911_H
#define __t_c911_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: t-plx9060.h,v 1.7 1997/09/11 21:58:36 steve Exp $"
/*
 * $Log: t-plx9060.h,v $
 * Revision 1.7  1997/09/11 21:58:36  steve
 *  Port the uCR device driver to Windows NT,
 *  make the configure select the target board
 *  and target host interface headers.
 *
 * Revision 1.6  1997/06/16 05:14:49  steve
 *  New channel protocol design and implementation.
 *
 * Revision 1.5  1997/02/25 20:37:52  steve
 *  Port back to i386. Now alpha and i386 work.
 *
 * Revision 1.4  1997/02/21 21:18:29  steve
 *  Port driver to Linux/Alpha.
 *
 * Revision 1.3  1996/10/28 03:46:57  steve
 *  License notice.
 *
 * Revision 1.2  1996/10/04 23:34:28  steve
 *  Add a BUS driver for the ISE board.
 *
 * Revision 1.1  1996/09/05 00:58:21  steve
 *  Move target-specific stuff into config header.
 *
 */

/*
 * The cyclone has on board a PLX chip for interface with the PCI bus,
 * and in turn ME. The base address of the chip is guessed from the
 * BIOS, and from there I use the following offsets to get/set
 * interesting fields of the chip. Use only the get and set functions
 * to manipulate the fields, and this driver will remain very
 * portable.
 */
enum PLXOffsets {
      MAILBOX0 = 0x40,
      MAILBOX1 = 0x44,
      MAILBOX2 = 0x48,
      MAILBOX3 = 0x4c,
      BELLS_OUT = 0x60,
      BELLS_IN  = 0x64,

      INTR_CNTRL= 0x68,

      DEVICE_SIZE = 0x70
};



/*
 * This method is called to initialize the hardware in preparation for
 * all the other calls. This should clear any interrupts and set
 * things up.
 */

static inline void dev_init_hardware(unsigned long plx)
{
      unsigned val = readl(plx+INTR_CNTRL);
      writel(val | 0x00000300, plx+INTR_CNTRL);
      writel(0xffffffff, plx+BELLS_IN);
}

/*
 * This turns off interrupts, so the board can no longer abuse the
 * host. It need not totally reset the board.
 */
static inline void dev_clear_hardware(unsigned long plx)
{
      unsigned val = readl(plx+INTR_CNTRL);
      writel(val & ~0x0000ff00, plx+INTR_CNTRL);
}

/*
 * The driver uses 3 mailbox registers to communicate with the target
 * board. It uses these methods to call them by name, and these
 * methods in turn dispatch them to the chosen registers. The
 * registers are:
 *
 *	dma_addr  -- DMA address word (out)
 *	dma_count -- DMA count and controlls word (out)
 *	status    -- STATUS word (in)
 */

static inline void dev_set_dma_addr(unsigned long plx, __u32 value)
{ writel(value, plx+MAILBOX0); }

static inline void dev_set_dma_count(unsigned long plx, __u32 value)
{ writel(value, plx+MAILBOX1); }

static inline __u32 dev_get_status(unsigned long plx)
{ return readl(plx+MAILBOX2); }

static inline __u32 dev_get_status_arg(unsigned long plx)
{ return readl(plx+MAILBOX3); }


/*
 * This method sends the bell interrupts that are set in the
 * mask. This is done by writing into the out bells register, which is
 * different from the in bells.
 */
static inline void dev_set_bells(unsigned long plx,
				 unsigned long mask)
{ writel(mask, plx+BELLS_OUT); }


/*
 * This method gets the bits of the currently set input bells, and
 * leaves those bits cleared in the bell register. That way, repeated
 * calls to dev_get_bells() will get a 1 only for bells that have been
 * set since the last call.
 */
static inline unsigned long dev_get_bells(unsigned long plx)
{
      unsigned long mask = readl(plx+BELLS_IN);
      writel(mask, plx+BELLS_IN);
      return mask;
}


/*
 * the dev_mask_irqs function masks the doorbell interrupts and
 * returns the current (before the mask) flag bits. The caller is
 * expected to pass that result back into the dev_unmask_irqs function
 * to restore interrupts.
 */
static __inline__ unsigned long dev_mask_irqs(unsigned long dev)
{
      unsigned long result = readl(dev+INTR_CNTRL);
      writel(result & ~0x300, dev+INTR_CNTRL);
      return result;
}

static __inline__ void dev_unmask_irqs(unsigned long dev, unsigned long mask)
{
      writel(readl(dev+INTR_CNTRL) | (mask&0x300), dev+INTR_CNTRL);
}

/*
 * Every PCI board type has a code to identify it. The target specific
 * code must define these.
 */
# define VENDOR_ID 0x113c
# define DEVICE_ID 0x0911
# define DEVICE_NAME "cyclone"
# define DEVICE_LONG_NAME "Cyclone 911"

#endif
